<?php

namespace HCM\FPA\Calculos\Contabilizacao;

use Doctrine\ORM\EntityManager;
use HCM\FPA\Entity\FpaValorreferenciamEntity;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\Util;
use HCM\Util\MsgCalculo;
use HCM\Util\FPA;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;
use HCM\FPA\Entity\FpaeventomEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FparesumoctblEntity;
use HCM\FPA\Entity\FparesumodroEntity;
use HCM\FPA\Entity\FpaeventoctblEntity;
use HCM\FPA\Entity\FpatipoassoceveEntity;
use HCM\Geral\Entity\HistpadrEntity;
use HCM\Geral\Entity\ContctblEntity;
use HCM\Geral\Entity\GpemovimentacaoEntity;
use HCM\Geral\Entity\EstruturamEntity;
use HCM\FPA\Entity\FpaTpdemissaoEntity;

class Contabilizacao {

    private $_nrorg;
    private $_nrOrgPadrao;
    private $_cdOperador;
    private $_simulacao;
    private $_mes_competencia;

    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;

    private $_formulario;
    
    private $_evento;
    private $_itens_calculo;
    private $_totaliza_eventos;
    private $_resumo_contabil;
    private $_resumo_dro;
    private $_evento_contabil;
    private $_historicoPadrao;
    private $_contaContabil;

    private $_lancamentos;
    private $_ult_nrlancamento;

    private $_estruturam;

    private $_movimentacao;

    private $_paramOrg;

    private $_tpDemissao;

    private $_nrtipomovimento;

    private $_idFormaContab;

    private $_arrIdFormaContab = array(
                'C' => true, 'T' => true
            );
    private $_arrIdFormaDespes = array(
                'D' => true, 'T' => true
            );

    private $_arrDespVinc = array();
    
    private $_totalCredito;
    private $_totalDebito;
    private $_totalDespesas;
    private $_registroContabilizacao;
    private $_registroResumoDespesas;
    
    private $_tipoContabilizacao;

    public function __construct($entityManager, $hcmParameters, $simulacao, $dataSet){
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrorg = $hcmParameters->getNrorgTrabalho();
        $this->_nrOrgPadrao = $hcmParameters->getNrorgPadrao();
        $this->_cdOperador = $hcmParameters->getCdoperador();
        $this->_simulacao = $simulacao;
        $this->_mes_competencia = $hcmParameters->getDtCompetencia();
        $this->dtInicio = DateUtil::getDataDeString($hcmParameters->getDtCompetencia(), true);
        $this->dtFim = DateUtil::getDataDeString($hcmParameters->getUltimoDiaMes(), true);
        $this->dtAux = DateUtil::getDataDeString($hcmParameters->getDataAtual(), true);
        $this->_lancamentos = array();
        $this->_ult_nrlancamento = 0;
        $this->_paramOrg = FPA::getGpeParametrosOrganizacao();
        $this->_evento = new FpaeventomEntity($entityManager, $hcmParameters);
        $this->itemCalculo = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
        $this->_resumo_contabil = new FparesumoctblEntity($entityManager, $hcmParameters);
        $this->_resumo_dro = new FparesumodroEntity($entityManager, $hcmParameters);
        $this->_evento_contabil = new FpaeventoctblEntity($entityManager, $hcmParameters);
        $this->_historicoPadrao = new HistpadrEntity($entityManager, $hcmParameters);
        $this->_contaContabil = new ContctblEntity($entityManager, $hcmParameters);
        $this->_movimentacao = new GpemovimentacaoEntity($entityManager, $hcmParameters);
        $this->_estruturam = new EstruturamEntity($entityManager, $hcmParameters);
        $this->_tpDemissao = new FpaTpdemissaoEntity($entityManager, $hcmParameters);
        $this->_inicializaVariaveis($dataSet);
        $this->_registroContabilizacao = null;
        $this->_totalCredito = null;
        $this->_totalDebito = null;
        $this->_registroResumoDespesas = null;
        $this->_totalDespesas = null;
    }

    private function _inicializaVariaveis($dataSet){

        $this->_nrtipomovimento = $dataSet['NRTIPOMOVIMENT'];
        $this->_idFormaContab = $dataSet['IDFORMACONTAB'];
        
        /*
         * Criação do valor de referência TIPO_CONTABILIZACAO para retirar as exceções fixas em código para as orgs 20, 1024 e 2025
         * Tipos de contabilização conforme o valor de TIPO_CONTABILIZACAO:
         * - 1: Contabiliza gerando o resumo contábil somente pela estrutura superior
         * - 2: Contabiliza gerando o resumo contábil pelas estrutura superior e estrutura relacionada (Normalmente para organizações que tem uma estrutura gerencial para várias estruturas legais. Ex: 2025)
         * - outros valores ou sem o valor de referência: Contabiliza gerando o resumo contábil de acordo com a hierarquia olhando somente para estrutura relacionada
         **/
        $this->_tipoContabilizacao = $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)->
            buscavalor($this->_nrorg, $this->_nrOrgPadrao, $this->_mes_competencia, 'TIPO_CONTABILIZACAO');
    }

    public function contabilizar($vinculo, array $lista_calculo_folha, $nr_tipo_hierarquia = null) {
         try {
            
            $nr_estrutura_sup = $this->_determinaNrEstruturaSuperior($vinculo);
            
            if ($this->_tipoContabilizacao == 1) {
                $nr_estrutura_relac = $nr_estrutura_sup;
            } else {
                $nr_estrutura_relac = $this->_determinaNrEstruturaRelacionada($vinculo);
            }
            
            $this->_totaliza_eventos = array();
            
            // Totaliza o array de eventos do vinculo.
            $totalizou = $this->_totalizaResumoEventos($vinculo, $lista_calculo_folha);
            
            if ($totalizou === true) {
                $this->_arrDespVinc = array();
                
                // Ja retorna as lista de resumos contabeis
                //if ($this->_nrorg == 20) {
                if ($this->_tipoContabilizacao == 1) {
                    if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
                        $cont = $this->_resumo_contabil->criaListaResumoContabilPorEstruturaSup(
                            $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac, $this->_nrtipomovimento
                        );
                    }
                    if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
                        $cont = $this->_resumo_dro->criaListaResumoDroPorEstruturaSup(
                            $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac, $this->_nrtipomovimento
                        );
                    }
                //} else if ($this->_nrorg == 1024 || $this->_nrorg == 2025) {
                } else if ($this->_tipoContabilizacao == 2) {
                    if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
                        $cont = $this->_resumo_contabil->criaListaResumoContabilPorEstruturas(
                            $this->_nrorg, $this->_mes_competencia, $nr_estrutura_sup, $nr_estrutura_relac, $this->_nrtipomovimento
                        );
                    }
                    if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
                        $cont = $this->_resumo_dro->criaListaResumoDroPorEstruturas(
                            $this->_nrorg, $this->_mes_competencia, $nr_estrutura_sup, $nr_estrutura_relac, $this->_nrtipomovimento
                        );
                    }
                } else {
                    if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
                        $cont = $this->_resumo_contabil->criaListaResumoContabil(
                            $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac, $this->_nrtipomovimento
                        );
                    }
                    if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
                        $cont = $this->_resumo_dro->criaListaResumoDro(
                            $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac, $this->_nrtipomovimento
                        );
                    }
                }

                $evento_contabil = null;
                
                // Percorre array de eventos totalizados persistindo os eventos associados a estrutura.
                foreach ($this->_totaliza_eventos as $nr_evento => $valor_evento) {
                    $data_rescisao_vinculo = $vinculo->getDtrescisaovinc();

                    // Só gera os eventos 2050 e 2080 se o tipo de rescisão é J (Pedido de Demissão ou Antecipação de Contrato por Parte do Empregado)
                    if ($this->_nrorg == 20) {
                        if ($nr_evento === 2150 || $nr_evento === 2140 || $nr_evento === FpatipoassoceveEntity::ASSOC_FGTS_RESC_DEPOSITADO || $nr_evento === FpatipoassoceveEntity::ASSOC_FGTS_13_RESCISAO) {
                            $objTpDemissao = $this->_tpDemissao->procuraTipoDemissao($vinculo->getNrtpdemissao());
                            if (is_object($objTpDemissao)) {
                                if ($objTpDemissao->getCodfgts() != 'J') {
                                    continue;
                                }
                            }
                        }
                    }
                    
                    // Pesquisa contas contabeis.
                    $evento_contabil = $this->_pesquisaContasContabeis(
                        $data_rescisao_vinculo, $nr_evento, $nr_estrutura_sup, $nr_estrutura_relac
                    );
                    
                    // Se o evento nao estiver associado a contabilizacao, pega o proximo evento.
                    if (is_object($evento_contabil) === false) {
                        continue;
                    } else {
                        $rateiovinc = $this->entityManager->getRepository(Repositories::FPA_ITRATEIOVINC)->getRateioVinculo($this->_nrorg, $vinculo->getNrvinculom(), $this->_mes_competencia);
                        if ($rateiovinc) {
                            foreach ($rateiovinc as $itemrateio) {
                                $valor_evento_rateio = round($valor_evento * $itemrateio['PERCENT'] / 100, 2);
                                $this->_atualizaResumoContabilDro($vinculo, $nr_estrutura_sup, $itemrateio['NRESTRUTURAM'], $nr_evento, $valor_evento_rateio, $evento_contabil);
                            }
                        } else {
                            $this->_atualizaResumoContabilDro($vinculo, $nr_estrutura_sup, $nr_estrutura_relac, $nr_evento, $valor_evento, $evento_contabil);
                        }
                    }

                    $competencia = DateUtil::getDataDeString($this->_mes_competencia);

                    $calc_folhaAtual = $this->entityManager
                        ->getRepository(Repositories::FPA_CALCULOFOLHA)
                        ->retornaCalculoFolha(
                            $this->_nrorg,
                            4,
                            1,
                            1,
                            $competencia,
                            $competencia)[0];

                    $itemCalcCompetenciaAtual = new FpaitecalcfolhaEntity(
                        $this->entityManager,
                        $this->hcmParameters,
                        $calc_folhaAtual->getNrcalculofolha());
                    $itemCalcCompetenciaAtual->retornaItensPorVinculo(
                        $vinculo->getNrorg(),
                        $vinculo->getNrvinculom(),
                        $calc_folhaAtual->getNrcalculofolha());
                    
                    $itensParaVerificar = $itemCalcCompetenciaAtual->getItems();
                    
                    foreach ($itensParaVerificar as $item) {
                        if ($item->getNreventom() == $nr_evento && $item->getIdevendemoncalc() == 'S') {
                            $this->_resumo_contabil->removeItensDemonstrativo13(
                                $evento_contabil->getCdcontctblcre(),
                                $evento_contabil->getCdcontctbldeb()
                            );
                            $this->_resumo_dro->removeItensDemonstrativo13(
                                $evento_contabil->getCddespadmfat()
                            );
                        }
                    }
                }
                
                $this->_resumo_contabil->removeItensZerados();
                
                if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
                    foreach ($this->_arrDespVinc as $cdDespadmfat) {
                        $this->_resumo_dro->atualizaListaResumoDro(
                            $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac,
                            $cdDespadmfat, 0, $nr_estrutura_sup, $this->_nrtipomovimento, 1
                        );
                    }
                }

                $this->_resumo_dro->removeItensZerados();
                
                /*//Tipo 1 
                if ($this->_tipoContabilizacao == 1) {
                    
                    $eventos_contabil = $this->retornaEventosTipo1();

                    if($eventos_contabil){
                        $nrLancamento = count($eventos_contabil) + 10;
                        foreach($eventos_contabil as $eventoContabil){
                            
                            //Insere credito
                            $this->_resumo_contabil->insereResumoContabil($this->_nrorg, DateUtil::getDataDeString($this->_mes_competencia),
                            $eventoContabil["ESTRUT"], 'C', $eventoContabil["CONTC"], $eventoContabil["HISTC"], $eventoContabil["TOTALC"], $eventoContabil["ESTRUTC"], $nrLancamento);
                            
                            //Insere Debito
                            $this->_resumo_contabil->insereResumoContabil($this->_nrorg, DateUtil::getDataDeString($this->_mes_competencia),
                            $eventoContabil["ESTRUT"], 'D', $eventoContabil["CONTD"], $eventoContabil["HISTD"], $eventoContabil["TOTALD"], $eventoContabil["ESTRUTC"], $nrLancamento);
                            $nrLancamento++;
                        }
                        $this->deleteEventosTipo1(count($eventos_contabil));
                    }
                }*/
                
            }
         } catch (\Exception $e) {
             throw $e;
         }

    }
    
    public function ajusteContabilizacaoTipo1(){
        if ($this->_tipoContabilizacao == 1) {
                    
            $eventos_contabil = $this->retornaEventosTipo1();
            
            if($eventos_contabil){
                $nrLancamento = count($eventos_contabil) + 10;
                foreach($eventos_contabil as $eventoContabil){
                    //Insere credito
                    $this->_resumo_contabil->insereResumoContabil($this->_nrorg, DateUtil::getDataDeString($this->_mes_competencia),
                    $eventoContabil["ESTRUT"], 'C', $eventoContabil["CONTC"], $eventoContabil["HISTC"], $eventoContabil["TOTALC"], $eventoContabil["ESTRUTC"], $nrLancamento, $this->_nrtipomovimento);
                    
                    //Insere Debito
                    $this->_resumo_contabil->insereResumoContabil($this->_nrorg, DateUtil::getDataDeString($this->_mes_competencia),
                    $eventoContabil["ESTRUT"], 'D', $eventoContabil["CONTD"], $eventoContabil["HISTD"], $eventoContabil["TOTALD"], $eventoContabil["ESTRUTC"], $nrLancamento, $this->_nrtipomovimento);
                    $nrLancamento++;
                }
                $this->deleteEventosTipo1(count($eventos_contabil));
            }
        }
    }
    
    public function deleteEventosTipo1($nrlancamento){
        
        $nrlancamento++;
        
        $query = $this->entityManager->getConnection()->prepare("
          DELETE FROM FPA_RESUMOCTBL WHERE NRORG = :P_NRORG AND NRLANCAMENTO < :P_NRLANCAMENTO AND DTOCORRENCIA = :P_COMPETENCIA AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT
        ");
        
        $return = $query->execute(
            array(
                "P_NRORG"            => $this->_nrorg,
                "P_NRLANCAMENTO" => $nrlancamento,
                "P_COMPETENCIA" => $this->_mes_competencia,
                "P_NRTIPOMOVIMENT" => $this->_nrtipomovimento
            )
        );
    }
    
    public function retornaEventosTipo1(){
        
        $query = "SELECT 
            C.NRESTRUTURAM ESTRUT, 
            C.NRESTRUTURAMSUP ESTRUTC, 
            C.CDCONTCTBL CONTC, 
            C.CDHISTPADR HISTC,
            D.NRESTRUTURAMSUP ESTRUTD, 
            D.CDCONTCTBL CONTD, 
            D.CDHISTPADR HISTD,
            MIN(D.NRLANCAMENTO) LANCAMENTOD,
            MIN(C.NRLANCAMENTO) LANCAMENTOC,
            SUM(C.VRLANCAMENTO) TOTALC,
            SUM(D.VRLANCAMENTO) TOTALD
           FROM
           (SELECT NRORG, DTOCORRENCIA, NRESTRUTURAMSUP, CDCONTCTBL, CDHISTPADR, NRLANCAMENTO, VRLANCAMENTO, NRESTRUTURAM
             FROM FPA_RESUMOCTBL
            WHERE NRORG = :P_NRORG
              AND DTOCORRENCIA = :P_DTMESCOMPETENC
              AND IDTIPOLANC = 'C'
              AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT) C,
           (SELECT NRORG, DTOCORRENCIA, NRESTRUTURAMSUP, CDCONTCTBL, CDHISTPADR, NRLANCAMENTO, VRLANCAMENTO
             FROM FPA_RESUMOCTBL
            WHERE NRORG = :P_NRORG
              AND DTOCORRENCIA = :P_DTMESCOMPETENC
              AND IDTIPOLANC = 'D'
              AND NRTIPOMOVIMENT = :P_NRTIPOMOVIMENT) D
           WHERE C.NRORG = :P_NRORG
             AND C.DTOCORRENCIA = :P_DTMESCOMPETENC
             AND D.NRORG = :P_NRORG
             AND D.DTOCORRENCIA = :P_DTMESCOMPETENC
             AND C.NRESTRUTURAMSUP = D.NRESTRUTURAMSUP
             AND C.NRLANCAMENTO = D.NRLANCAMENTO
             AND C.CDHISTPADR = D.CDHISTPADR
        GROUP BY 
            C.NRESTRUTURAM,
            C.NRESTRUTURAMSUP, 
            C.CDCONTCTBL, 
            C.CDHISTPADR,
            D.NRESTRUTURAMSUP, 
            D.CDCONTCTBL, 
            D.CDHISTPADR
        ORDER BY
            HISTC,
            LANCAMENTOC";
            
            $params = array("P_NRORG"           => $this->_nrorg,
                        "P_DTMESCOMPETENC"  => $this->_mes_competencia,
                        "P_NRTIPOMOVIMENT" => $this->_nrtipomovimento);
            
            $db = \Setforms2\Shared\Registry::getEntityManagerConnection();
            
        return $db->fetchAll($query, $params);
    }
    
    public function _atualizaResumoContabilDro($vinculo, $nr_estrutura_sup, $nr_estrutura_relac, $nr_evento, $valor_evento, $evento_contabil) {
        
        if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
            if(!is_null($evento_contabil->getCdcontctblcre()) || !is_null($evento_contabil->getCdhistpadrcre()) || !is_null($evento_contabil->getCdcontctbldeb()) || !is_null($evento_contabil->getCdhistpadrdeb())) {
                // Busca/Gera número do lançamento
                
                $nrlancamento = $this->getNrlancamento($nr_estrutura_sup, $nr_estrutura_relac, $nr_evento, $evento_contabil->getIdvincativo());
        
                /* Pesquisa no resumo se ja existe conta de debito naquele mes de competencia e para aquela
                 * estrutura.
                 */
                $this->_resumo_contabil->atualizaListaResumoContabil(
                    $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac,
                    FparesumoctblEntity::IDTIPOLANC_DEBITO, $evento_contabil->getCdcontctbldeb(),
                    $evento_contabil->getCdhistpadrdeb(), $valor_evento, $nr_estrutura_sup, $nrlancamento,
                    $this->_nrtipomovimento
                );
        
                /* Pesquisa no resumo se ja existe conta de credito naquele mes de competencia e para aquela
                 * estrutura.
                 */
                $this->_resumo_contabil->atualizaListaResumoContabil(
                    $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac,
                    FparesumoctblEntity::IDTIPOLANC_CREDITO, $evento_contabil->getCdcontctblcre(),
                    $evento_contabil->getCdhistpadrcre(), $valor_evento, $nr_estrutura_sup, $nrlancamento,
                    $this->_nrtipomovimento
                );
            }
        }
        
        if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
    
            $evento = $this->_evento->getEvento($nr_evento);
            
            // Se for um evento de desconto, subtrai no valor total da despesa
            if ($evento->getIdtipoevento() == FpaeventomEntity::IDTIPOEVENTO_DESCONTO) {
                $valor_evento = $valor_evento * -1;
            }
            /* Pesquisa no resumo se ja existe despesa naquele mes de competencia e para aquela
             * estrutura.
             */
            $this->_resumo_dro->atualizaListaResumoDro(
                $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac,
                $evento_contabil->getCddespadmfat(), $valor_evento, $nr_estrutura_sup,
                $this->_nrtipomovimento
            );
    
            /* Inclui despesa no array para somar a quantidade de vínculos por despesa */
            if (array_search($evento_contabil->getCddespadmfat(), $this->_arrDespVinc) === false) {
                $this->_arrDespVinc[] = $evento_contabil->getCddespadmfat();
            }
        }
    }

    public function contabilizarLite($dadosContabilizacao, $dadosDespesas) {
        try {
            $valores_referencia = new FpaValorreferenciamEntity($this->entityManager, $this->hcmParameters);
            $valorReferenciaTipoContabilizacao = $valores_referencia->getValorReferencia('TIPO_CONTABILIZACAO');

            if ($valorReferenciaTipoContabilizacao == 1) {
                throw new \Exception('Para sua parametrização a "Gravação Definitiva Otimizada" não está disponível. Por favor utilize a "Gravação Definitiva"');
            }

            if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
                foreach ($dadosContabilizacao as $index => $dados) {
                    $nrlancamento = ($index + 1);
                    $nr_estrutura_sup = $dados['NRESTRUTURAMSUP'];
                    $nr_estrutura_relac = $dados['NRESTRUTURAM'];
                    $cdcontctbldeb = $dados['CDCONTCTBLDEB'];
                    $cdhistpadrdeb = $dados['CDHISTPADRDEB'];
                    $cdcontctblcre = $dados['CDCONTCTBLCRE'];
                    $cdhistpadrcre = $dados['CDHISTPADRCRE'];
                    $vrlancamento = $dados['VRLANCAMENTO'];
                    
                    if (empty($this->_resumo_contabil->getItems())) {
                        $cont = $this->_resumo_contabil->criaListaResumoContabilLite(
                            $this->_nrorg, $this->_mes_competencia, $this->_nrtipomovimento
                        );
                    }

                    $this->_resumo_contabil->atualizaListaResumoContabil(
                        $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac,
                        FparesumoctblEntity::IDTIPOLANC_DEBITO, $cdcontctbldeb,
                        $cdhistpadrdeb, $vrlancamento, $nr_estrutura_sup, $nrlancamento,
                        $this->_nrtipomovimento
                    );

                    $this->_resumo_contabil->atualizaListaResumoContabil(
                        $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac,
                        FparesumoctblEntity::IDTIPOLANC_CREDITO, $cdcontctblcre,
                        $cdhistpadrcre, $vrlancamento, $nr_estrutura_sup, $nrlancamento,
                        $this->_nrtipomovimento
                    );
                }
            }

            if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
                foreach ($dadosDespesas as $index => $dados) {
                    $nr_estrutura_sup = $dados['NRESTRUTURAMSUP'];
                    $nr_estrutura_relac = $dados['NRESTRUTURAM'];
                    $vrlancamento = $dados['VRLANCAMENTO'];
                    $cddespadmfat = $dados['CDDESPADMFAT'];

                    if (empty($this->_resumo_dro->getItems())) {
                        $this->_resumo_dro->criaListaResumoDroLite(
                            $this->_nrorg, $this->_mes_competencia, $this->_nrtipomovimento
                        );
                    }

                    $this->_resumo_dro->atualizaListaResumoDro(
                        $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac,
                        $cddespadmfat, $vrlancamento, $nr_estrutura_sup,
                        $this->_nrtipomovimento
                    );
                }
            }
            
            $this->_resumo_contabil->removeItensZerados();
            $this->_resumo_dro->removeItensZerados();
            
            foreach ($dadosContabilizacao as $contabilizacao) {
                if ($contabilizacao["IDEVENDEMONCALC"] == 'S') {
                    $this->_resumo_contabil->removeItensDemonstrativo13(
                        $contabilizacao['CDCONTCTBLCRE'],
                        $contabilizacao['CDCONTCTBLDEB']
                    );
                }
            }

            foreach ($dadosDespesas as $despesa) {
                if ($despesa["IDEVENDEMONCALC"] == 'S') {
                    $this->_resumo_dro->removeItensDemonstrativo13(
                        $despesa['CDDESPADMFAT']
                    );
                }
            }
         } catch (\Exception $e) {
             throw $e;
         }

    }
    
    private function getNrlancamento($nr_estrutura_sup, $nr_estrutura_relac, $nr_evento, $idvincativo)
    {
        //Verifica se existe no array lançamentos se a chave de numero estrutura sup
        if (array_key_exists($nr_estrutura_sup, $this->_lancamentos)) {
            //verifica se existe no array de lançamentos['nr estrutura sup'] uma esturura relacional
            if (array_key_exists($nr_estrutura_relac, $this->_lancamentos[$nr_estrutura_sup])) {
                //verifica se existe no array de lançamentos['nr estrutura sup'] e dentro ['nresturura relacional' o numero de evento] 
                if (array_key_exists($nr_evento . $idvincativo, $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac])) {
                    return $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac][$nr_evento . $idvincativo];
                }
                //Se não existir
                else {
                    $this->_ult_nrlancamento++;
                    $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac][$nr_evento . $idvincativo] = $this->_ult_nrlancamento;
                    return $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac][$nr_evento . $idvincativo];
                }
            }
            //Se não existir
            else {
                $this->_ult_nrlancamento++;
                $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac][$nr_evento . $idvincativo] = $this->_ult_nrlancamento;
                return $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac][$nr_evento . $idvincativo];
            }
        //Se não tiver nenhum lançamento
        } else {
            //Adiciona um nr de lançamento se não existir nenhum
            $this->_ult_nrlancamento++;
            $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac][$nr_evento . $idvincativo] = $this->_ult_nrlancamento;
            return $this->_lancamentos[$nr_estrutura_sup][$nr_estrutura_relac][$nr_evento . $idvincativo];
        }
    }

    public function retrocedeResumoEventos(
        $vinculo
    ){
        $nr_estrutura_relac = $this->_determinaNrEstruturaRelacionada($vinculo);
        $nr_estrutura_sup = $this->_determinaNrEstruturaSuperior($vinculo);

        if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
            //Retrocede para a estrutura legal.
            /*
                $gerouLista = $this->_resumo_contabil->criaListaResumoContabil(
                    $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac, $this->_nrtipomovimento
                );
            */
            
            $gerouLista = $this->_resumo_contabil->criaListaResumoContabilPorEstruturaSup(
                $this->_nrorg, $this->_mes_competencia, $nr_estrutura_sup, $this->_nrtipomovimento
            );
            
            if ($gerouLista === true) {
                $retrocede = $this->_resumo_contabil->retrocedeListaResumoContabil();
                if($retrocede){
                    $this->_resumo_contabil->persist();
                }
            }
        }
        if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
            //Retrocede para a estrutura legal.
            /*
                $gerouListaDro = $this->_resumo_dro->criaListaResumoDro(
                    $this->_nrorg, $this->_mes_competencia, $nr_estrutura_relac, $this->_nrtipomovimento
                );
            */
            
            $gerouListaDro = $this->_resumo_dro->criaListaResumoDroPorEstruturaSup(
                $this->_nrorg, $this->_mes_competencia, $nr_estrutura_sup, $this->_nrtipomovimento
            );
                
            if ($gerouListaDro === true) {
                $retrocede = $this->_resumo_dro->retrocedeListaResumoDro();
                if($retrocede){
                    $this->_resumo_dro->persist();
                }
            }
        }
        if($retrocede){
            return true;
        }else{
            return false;
        } 
    }

    public function _determinaNrEstruturaSuperior(
        $vinculo
    ){
        $movimentacao_recente = $this->_movimentacao->retornaMovimentacaoRecentePorEstrutura(
            $this->_nrorg, $this->_paramOrg->getNrtpestrutlegal(), $vinculo->getNrvinculom(), $this->_mes_competencia
        );
        
        if ($movimentacao_recente === false) {
            $msg = 'Movimentacao inexistente para o vínculo '.$vinculo->getNrvinculom();
            throw new \Exception($msg);
        }
        
        if ($vinculo->historico->getNrestrutlegal() != $movimentacao_recente->getNrestruturam()) {
            $msg = 'Vínculo ' . $vinculo->getNrvinculom() . ' com movimentação inválida para a estrutura ' . $movimentacao_recente->getNrestruturam();
            throw new \Exception($msg);
        }

        return $movimentacao_recente->getNrestruturam();
    }

    public function _determinaNrEstruturaRelacionada(
        $vinculo
    ){
        $movimentacao_recente = $this->_movimentacao->retornaMovimentacaoRecentePorEstrutura(
            $this->_nrorg, $this->_paramOrg->getNrtpestrutgeren(), $vinculo->getNrvinculom(), $this->_mes_competencia
        );
        
        if ($movimentacao_recente === false) {
            $msg = 'Movimentacao inexistente para o vínculo '.$vinculo->getNrvinculom();
            throw new \Exception($msg);
        }

        if ($vinculo->historico->getNrestrutgeren() != $movimentacao_recente->getNrestruturam()) {
            $msg = 'Vínculo ' . $vinculo->getNrvinculom() . ' com movimentação inválida para a estrutura ' . $movimentacao_recente->getNrestruturam();
            throw new \Exception($msg);
        }

        return $movimentacao_recente->getNrestruturam();
    }

    private function _totalizaResumoEventos($vinculo, array $lista_calculo_folha){
        // Percorre os numeros de calculo totalizando os eventos.
        foreach ($lista_calculo_folha as $nr_calculo_folha) {
            $quantidade_itens = $this->itemCalculo->retornaItensPorVinculo(
                $vinculo->getNrorg(), $vinculo->getNrvinculom(), $nr_calculo_folha
            );
            /* @todo parar o loop no calculo de folha ativo caso nao tenha itens (verificar) */
            if ($quantidade_itens === 0) {
                continue;
            }

            $itens = $this->itemCalculo->getItems();
            
            $resultadoCalcFolha = $this->entityManager->getRepository(Repositories::FPA_CALCULOFOLHA)->findOneBy(array('nrorg' => $vinculo->getNrorg(),'nrcalculofolha' => $nr_calculo_folha));
            
            foreach ($itens as $item) {
                if($item->getVreventofolha() > 0 && (($resultadoCalcFolha->getNrtipomoviment() == 4 && $item->getIdevendemoncalc() != 'S') || ($resultadoCalcFolha->getNrtipomoviment() != 4 && $item->getIdevendemoncalc() != 'S'))){
                    @$this->_totaliza_eventos[$item->getNreventom()] += $item->getVreventofolha();
                }
            }
        }

        if (empty($this->_totaliza_eventos)) {
            return false;
        } else {
            return true;
        }
    }

    private function _pesquisaContasContabeis($data_rescisao_vinculo, $nr_evento, $nr_estrutura_sup, $nr_estrutura_relac){

        $estruturaRelac = $this->_estruturam->retornaEstruturaPorNrestruturam($this->_nrorg, $nr_estrutura_relac);
        
        if ($data_rescisao_vinculo) {
            $evento_contabil = $this->_evento_contabil->pesquisaContasContabeis(
                $this->_nrorg, $nr_evento, $nr_estrutura_sup,
                $this->_mes_competencia, FpaeventoctblEntity::IDVINCATIVO_NAO,
                $estruturaRelac->getNrtpeventoctbl(), $this->_nrtipomovimento
            );
            if (is_object($evento_contabil) === false) {
                $evento_contabil = $this->_evento_contabil->pesquisaContasContabeis(
                    $this->_nrorg, $nr_evento, $nr_estrutura_sup,
                    $this->_mes_competencia, FpaeventoctblEntity::IDVINCATIVO_SIM,
                    $estruturaRelac->getNrtpeventoctbl(), $this->_nrtipomovimento
                );
            }
        } else {
            $evento_contabil = $this->_evento_contabil->pesquisaContasContabeis(
                $this->_nrorg, $nr_evento, $nr_estrutura_sup,
                $this->_mes_competencia, FpaeventoctblEntity::IDVINCATIVO_SIM,
                $estruturaRelac->getNrtpeventoctbl(), $this->_nrtipomovimento
            );
        }
        
        return $evento_contabil;
    }

    public function getTotalizaEventos(){
        return $this->_totaliza_eventos;
    }

    public function getResumoContabil() {
        return $this->_resumo_contabil;
    }

    public function getResumoDro() {
        return $this->_resumo_dro;
    }

    public function preparaDemonstrativo(){
        $demonstrativo = array();
        if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
            $itens = $this->_resumo_contabil->getItems();
            foreach ($itens as $item) {
                $estruturasup = $this->_estruturam->retornaEstruturaPorNrestruturam($this->_nrorg, $item->getNrestruturamsup());
                $dsContaContabil = $this->_contaContabil->getDescricaoContaContabil($this->_nrorg, $item->getCdcontctbl());
                $dsHistorico = $this->_historicoPadrao->getDescricaoHistorico($this->_nrorg, $item->getCdhistpadr());
                $estrutHistorico = $this->_estruturam->getHistorico($this->_nrorg,$item->getNrestruturamsup());
                $this->addRegistroContabilizacao(
                    $item->getNrestruturamsup(), $estrutHistorico->getNmestruturah(), $item->getCdcontctbl(), $dsContaContabil, $item->getCdhistpadr(), $dsHistorico, $item->getVrlancamento(), $item->getIdtipolanc()
                );
            }
            $demonstrativo['contabil'] = $this->_registroContabilizacao;
            $demonstrativo['credito'] = $this->_totalCredito;
            $demonstrativo['debito'] = $this->_totalDebito;
        }
        if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
            $itensDro = $this->_resumo_dro->getItems();
            foreach($itensDro as $item){
                $estruturasup = $this->_estruturam->retornaEstruturaPorNrestruturam($this->_nrorg, $item->getNrestruturamsup());
                $estrutura = $this->_estruturam->retornaEstruturaPorNrestruturam($this->_nrorg, $item->getNrestruturam());
                $despesa = \Faturamento\Entities\Despadmfatur::findOneByNrorgAndCddespadmfat($this->_nrorg, $item->getCddespadmfat());
                $dsDespesa = is_object($despesa) ? $despesa->getDsdespadmfat() : 's/ cód. despesa';
                $estrutHistorico = $this->_estruturam->getHistorico($this->_nrorg,$item->getNrestruturam());
                $estrutSupHistorico = $this->_estruturam->getHistorico($this->_nrorg,$item->getNrestruturamsup());
                $this->addRegistroResumoDespesas(
                    $item->getNrestruturamsup(), $estrutSupHistorico->getNmestruturah(), $item->getNrestruturam(), $estrutHistorico->getNmestruturah(), $item->getCddespadmfat(), $dsDespesa, $item->getVrlancamento()
                );
            }
            $demonstrativo['despesas'] = $this->_registroResumoDespesas;
            $demonstrativo['totaldes'] = $this->_totalDespesas;
        }
        return $demonstrativo;
    }
    
    public function addRegistroContabilizacao($nrEstruturaSup, $nmEstruturaSup, $nrConta, $dsConta, $nrHistorico, $dsHistorico, $valor, $tipo){
        
        if($tipo == 'C'){
            $this->_totalCredito += $valor;
        }elseif($tipo = 'D'){
            $this->_totalDebito += $valor;
        }
        
        if($tipo == 'C'){
            $tipo = "Conta de Crédito";
        }elseif($tipo = 'D'){
            $tipo = "Conta de Débito";
        }
        
        $this->_registroContabilizacao[] = array('ESTRUTURASUP' => $nrEstruturaSup.' '.$nmEstruturaSup, 'CONTA' => $nrConta.' '.$dsConta, 'TIPO' => $tipo, 'HISTORICO' => $nrHistorico.' '.$dsHistorico, 'VALOR' => number_format($valor, 2, ', ', '. '));
    }
    
    public function addRegistroResumoDespesas($nrEstruturaSup, $nmEstruturaSup, $nrEstrutura, $nmEstrutura, $cdDespesa, $dsDespesa, $valor){
        $this->_totalDespesas += $valor;
        $this->_registroResumoDespesas[] = array('ESTRUTURASUP' => $nrEstruturaSup.' '.$nmEstruturaSup, 'ESTRUTURA' => $nrEstrutura.' '.$nmEstrutura, 'DESPESA' => $cdDespesa.' '.$dsDespesa, 'VALOR' => number_format($valor, 2, ', ', '. '));
    }
    
    public function preparaEmail(){
        $email = array();
        if (isset($this->_arrIdFormaContab[$this->_idFormaContab])) {
            $itens = $this->_resumo_contabil->getItems();
            $email[] =  '                             Demonstrativo - Resumo Contábil'.PHP_EOL;
            foreach ($itens as $item) {
 
                $estruturasup = $this->_estruturam->retornaEstruturaPorNrestruturam($this->_nrorg, $item->getNrestruturamsup());
                $dsContaContabil = $this->_contaContabil->getDescricaoContaContabil($this->_nrorg, $item->getCdcontctbl());
                $dsHistorico = $this->_historicoPadrao->getDescricaoHistorico($this->_nrorg, $item->getCdhistpadr());

                if($item->getIdtipolanc() == 'C'){
                    $this->_totalCredito += $item->getVrlancamento();
                    $tipo = "Conta de Crédito";
                }elseif($item->getIdtipolanc() == 'D'){
                    $tipo = "Conta de Débito";
                    $this->_totalDebito += $item->getVrlancamento();
                }else{
                    $tipo = $item->getIdtipolanc();
                }

                $email[] =
                    $item->getNrestruturamsup().' '. $estruturasup->historico->getNmestruturah().'   '. $item->getCdcontctbl().' '. $dsContaContabil.'   '.$tipo.'   '. $item->getCdhistpadr().' '. $dsHistorico.'   '. number_format($item->getVrlancamento(), 2, ', ', '. ')
                ;
            }
            $email[] = PHP_EOL.PHP_EOL.'Total - Conta de Crédito:                                                         '.number_format($this->_totalCredito, 2, ', ', '. ');
            $email[] = PHP_EOL.PHP_EOL.'Total - Conta de Débito:                                                         '.number_format($this->_totalDebito, 2, ', ', '. ').PHP_EOL.PHP_EOL;
        }
        if (isset($this->_arrIdFormaDespes[$this->_idFormaContab])) {
            $itensDro = $this->_resumo_dro->getItems();
            $email[] =  '                             Demonstrativo - Resumo de Despesas'.PHP_EOL;
            foreach($itensDro as $item){
     
                $estruturasup = $this->_estruturam->retornaEstruturaPorNrestruturam($this->_nrorg, $item->getNrestruturamsup());
                $estrutura = $this->_estruturam->retornaEstruturaPorNrestruturam($this->_nrorg, $item->getNrestruturam());
                $despesa = $this->entityManager->getRepository(Repositories::DESPADMFATUR)->findOneBy(array('nrorg' => $this->_nrorg, 'cddespadmfat' => $item->getCddespadmfat()));
                $dsDespesa = is_object($despesa) ? $despesa->getDsdespadmfat() : 's/ cód. despesa';

                $this->_totalDespesas += $item->getVrlancamento();
                $email[] =
                    $item->getNrestruturamsup().' '. $estruturasup->historico->getNmestruturah().'    '. $item->getNrestruturam().' '. $estrutura->historico->getNmestruturah().'    '.
                    $item->getCddespadmfat().' '. $dsDespesa.'   '. number_format($item->getVrlancamento(), 2, ', ', '. ')
                ;
            }
            $email[] = PHP_EOL.PHP_EOL.'Total - Despesas:                                                         '.number_format($this->_totalDespesas, 2, ', ', '. ');

        }
        return $email;
    }
        
    public function persistResumoContabil() {
        if (count($this->_resumo_contabil->getItems()) > 0) {
            $this->_resumo_contabil->persist();
        }
    }

    public function persistResumoDro() {
        if (count($this->_resumo_dro->getItems()) > 0) {
            foreach($this->_resumo_dro->getItems() as $resumo) {
                // Utiliza o valor absoluto do total de despesas pois podem haver valores negativos
                $resumo->setVrlancamento($resumo->getVrlancamento());
            }
            $this->_resumo_dro->persist();
        }
    }
}