<?php
namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\PTO\Services\ApuracaoFrequenciaService;
use Folha\Logic\ControlePonto\Apuracao;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\DateUtil;
use HCM\Util\Messages;
use HCM\Util\AlertException;

class ApuracaoFrequenciaController extends Crud {

    protected $dataSourceGetApuracaoFrequencia          = 'getApuracaoFrequencia';
    protected $dataSourceSetApuracaoFrequencia          = 'gpeApuraFrequencia_setApuracaoFrequencia';
    protected $dataSourceGetAcompanhamentoPeriodo       = 'getAcompanhamentoPeriodo';
    protected $apuracaoFrequenciaService;

    const Alfabetica    = 'ALFABETICA';
    const Data_Admissao = 'DATA_ADMISSAO';
    const Matricula     = 'MATRICULA';
    const Estrutura     = 'ESTRUTURA';
    
    const QUANT_VINCULOS_SIMULACAO          = 5;
    const INDEX_INICIO_VINCULOS_SIMULACAO   = 0;
    
    const ORGS_APURACAO_LOCAL = [1664];

    protected $apuracoes;

    protected $parameterBagColumns = array('NRORG', 'NRORGPADRAO', 'DTMESCOMPETENC', 'NRPERIODOAPURACAO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, ApuracaoFrequenciaService $apuracaoFrequenciaService, FPA $fpa, HCMParameters $hcmParameters){
        $this->parameterBag    = $parameterBag;
        $this->apuracaoFrequenciaService = $apuracaoFrequenciaService;
        $this->fpa = $fpa;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getApuracaoFrequencia (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->parameterBagColumns = array('NRORG', 'NRORGPADRAO', 'DTMESCOMPETENC');
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetApuracaoFrequencia);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getPeriodoSimulacao (DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $parameters = array();
        foreach ($filters as $key => $filter) {
            if($filter['columnName'] == 'NRESTRUTLEGAL' || $filter['columnName'] == 'NRESTRUTLEGAL' || $filter['columnName'] == 'NRESTRUTGEREN' || $filter['columnName'] == 'NRVINCULOM'){
                if (empty($filter['value'])) {
                    unset($filters[$key]);
                    continue;
                }
                
                $filter['value']  = implode(',', $filter['value']);
            }
            $parameters['P_'.$filter['columnName']] = $filter['value'];
        }
        $parameters['P_DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();
        
        $result = $this->apuracaoFrequenciaService->getPeriodoSimulacao($parameters);
        
        if(!isset($result[0]) && !empty($result)) {
            $result = array($result);
        }
        //Seta o campo de NRPRIORIDADE nos dias do periodo selecinado para colorir as linhas de acordo com a advertência
        foreach($result as $key => $dia){
            $nrprioridade = $this->getNrPrioridade($dia);
            $dia['NRPRIORIDADE'] = $nrprioridade;
            $result[$key] = $dia;
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getPeriodoSimulacao', $result));
    }
    
    public function getNrPrioridade($diaT){
        $apuracoes =  $this->hcmParameters->getApuracoesFrequencia();
        $data      = DateUtil::getDataDeString($diaT['DATA'],
            DateUtil::FORMATO_BRASILEIRO,
            true);
            
        if(array_key_exists((string) $diaT['NRVINCULOM'], $apuracoes)){
            $diaPeriodo = $apuracoes[$diaT['NRVINCULOM']]->getDiaPeriodo($data);
            $dia        = $diaPeriodo['DIA'];
            $totalizacao = array();
            /** @var Dia $dia */
            if(is_object($dia)){
                $totalizacao = $dia->getDatasourceTotalizacao();
            }
            //faz a verificação da prioridade(1 > 2 > 3 ...)
            $nrprioridade = PHP_INT_MAX;
            $cor = false;
            foreach($totalizacao as $row){
                if(($row['NRPRIORIDADE']) && ($row['NRPRIORIDADE'] < $nrprioridade)){
                    $nrprioridade = $row['NRPRIORIDADE'];
                    $cor = true;
                }
            }
            return ($cor) ? $nrprioridade : null ;    
        }else{
            return null;
        }
    }

    public function getVinculosSimulacao (DTO\Request\Filter $request, DTO\Response $response) {
        $filters = $request->getFilterCriteria()->getConditions();
        $parameters = array();
        foreach ($filters as $key => $filter) {
            $parameters[$filter['columnName']] = $filter['value'];
            switch($parameters['DSORDEM']){
                case self::Alfabetica:
                    $parameters['CRITERIO'] = 'NMVINCULOM';
                    break;
                case self::Data_Admissao:
                    $parameters['CRITERIO'] = 'DTADMISSAOVINC';
                    break;
                case self::Matricula:
                    $parameters['CRITERIO'] = 'CDMATRICULA';
                    break;
                case self::Estrutura:
                    $parameters['CRITERIO'] = 'NRESTRUTLEGAL';
                    break;
                default:
                    $parameters['CRITERIO'] = 'TODOS';
                    break;
            }
        }
        $result = $this->apuracaoFrequenciaService->getVinculosSimulacao($parameters);
        if(!isset($result[0])) {
            $result = array($result);
        }
        $vinculos = $this->hcmParameters->getDadosSimulacao();
      
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getVinculosSimulacao', $vinculos['SIMULADOS']));
    }

    public function setApuracaoFrequencia (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetApuracaoFrequencia;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRAPURAFREQUENCIA'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'GPE_APURAFREQUENCIA', 1, 12, true);
                } else {
                    $row['DTULTATU'] = DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                
                $row['NRORG'] = $row['NRORGTRAB'];
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteApuracaoFrequencia (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetApuracaoFrequencia, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetApuracaoFrequencia, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getDadosFormulario($request) {
        $dataset = array();        
        if ($request->getRoutePath() === '/efetivarApuracao') { 
            $dataset = $request->getDataSet()->getRows()[0];
        } else if ($request->getRoutePath() === '/efetivarApuracaoSimulacao') {
            $dataset = $this->hcmParameters->getDadosSimulacao();
        }

        return $dataset;
    }

    public function efetivarApuracao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            set_time_limit(0);
            $dataset = $this->getDadosFormulario($request);
        
            $nrPeriodoApuracao  = $dataset['NRPERIODOAPURACAO'];
            $iniPeriodo         = str_replace(' 00:00:00', '', $dataset['DTINICIAL']);
            $fimPeriodo         = str_replace(' 00:00:00', '', $dataset['DTFINAL']);
            $ordem              = $dataset['DSORDEM'];
            $dsInicio           = $dataset['DSINICIO'];
            $dsFim              = $dataset['DSFIM'];
            $nrTipoVinculo      = $dataset['NRTIPOVINCULOM'];
            $nrOrg              = $dataset['NRORGTRAB'];
            $nrOrgPadrao        = $dataset['NRORGPADRAO'];
            $dtCompetencia      = $dataset['DTCOMPETENCIA'];
            $apuraRescindidos   = $dataset['IDAPURARESCINDIDOS'];
            
            $requestWebService = !in_array($nrOrg, self::ORGS_APURACAO_LOCAL);

            switch($ordem){
                case self::Alfabetica:
                    $criterio = 'NMVINCULOM';
                    break;
                case self::Data_Admissao:
                    $criterio = 'DTADMISSAOVINC';
                    break;
                case self::Matricula:
                    $criterio = 'CDMATRICULA';
                    break;
                case self::Estrutura:
                    $criterio = 'NRESTRUTLEGAL';
                    break;
                default:
                    $criterio = 'TODOS';
                    break;
            }

            $errorLog = null;
            $alertLog = null;

            $vinculos   = $this->apuracaoFrequenciaService->getVinculosPorSelecaoNRescindidos($criterio, $dsInicio, $dsFim, $dtCompetencia, $nrTipoVinculo, $nrOrgPadrao, $nrOrg, $apuraRescindidos, $fimPeriodo, $iniPeriodo);
            
            // Valida se foram encontrados vínculos
            if (!is_array($vinculos) || count($vinculos) < 1) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM07022_PTO_FILTRO_VINC_NOT_FOUND));
                return;
            }
            
            $vinculosNaoGravados = '';
            $vinculosSemControle = null;
            $vinculosGravados = null;
            $vinculosSemEscala = null;
            $vinculosApurados = null;
            $apurados = array();

            try{
                $apurado = array();
                foreach($vinculos as $vinculo) {
                    $verificaResumo = $this->apuracaoFrequenciaService->getResumoSituFreqVinculo($nrOrg, $vinculo['NRVINCULOM'], $nrPeriodoApuracao);
                    $idnaocontrolajornada = $this->apuracaoFrequenciaService->verificaControleJornada($nrOrg, $vinculo['NRVINCULOM'], $fimPeriodo);
                    if (!empty($idnaocontrolajornada)) {
                        if ($idnaocontrolajornada == 'N') {
                            if(empty($verificaResumo) || !is_array($verificaResumo)) {
                                
                                // Dentro desse bloco, tudo que é levantado (inclusive Exceptions e Errors) são tratados. Tratativa para que a apuração seja realizada para todos os vínculos selecionados
                                try {
                                    
                                    if ($requestWebService) {
                                        $this->apuracaoFrequenciaService->requestApuracaoWebService($vinculo['NRVINCULOM'], $nrPeriodoApuracao);
                                    } else {
                                        $apuracao = new Apuracao($nrOrg, $vinculo['NRVINCULOM'], $iniPeriodo, $fimPeriodo, $nrPeriodoApuracao);
                                        $apuracao->gravaApuracao();
                                    }
                                    
                                    if(!is_null($vinculosApurados)){
                                         $vinculosApurados.= '<br>';
                                    }
                                    $vinculosApurados .= $vinculo['NMVINCULOM'];
                                    array_push($apurado, $vinculo['NMVINCULOM']);
                                } catch (AlertException $ex) {
                                    $alertLog .= '<br><br>Alertas na apuração do vínculo '.$vinculo['NMVINCULOM'].' ('.$vinculo['NRVINCULOM'].'): '.$ex->getMessage();
                                } catch (\Exception $ex) {
                                    $errorLog .= '<br><br>Inconsistência na apuração do vínculo '.$vinculo['NMVINCULOM'].' ('.$vinculo['NRVINCULOM'].'): '.$ex->getMessage();
                                }

                            } else {
                                if(!is_null($vinculosGravados)){
                                 $vinculosGravados.= ' ';
                                }
                                $vinculosGravados.= $vinculo['NMVINCULOM'];
                            }
                        } else {
                            if(!is_null($vinculosSemControle)){
                             $vinculosSemControle.= ' ';
                            }
                            $vinculosSemControle.= $vinculo['NMVINCULOM'];
                        }
                    } else {
                        if(!is_null($vinculosSemEscala)){
                             $vinculosSemEscala.= ' ';
                        }
                        $vinculosSemEscala.= $vinculo['NMVINCULOM'];
                    }
                }  
                $message = null;
               
                if(!is_null($errorLog)){
                    if(!is_null($message)){
                        $message.= '<br><br>';
                    }
                    $message.= "<br>Foram encontradas inconsistências durante a apuração." . $errorLog;
                }
                if(!is_null($alertLog)){
                    if(!is_null($message)){
                        $message.= '<br><br>';
                    }
                    $message.= "<br>Foram encontrados alertas durante a apuração." . $alertLog;
                }
                if(!is_null($vinculosApurados)){
                    if(!is_null($message)){
                        $message.= '<br><br>';
                    }
                    $message.= "O(s) Vínculo(s) <br> ".$vinculosApurados." <br> foram apurados.\n";
                }
                if(!is_null($vinculosSemControle)){
                    if(!is_null($message)){
                        $message.= '<br><br>';
                    }
                    $message.= "O(s) Vínculo(s) <br> ".$vinculosSemControle." <br> não possui(em) controle de jornada.\n";
                }
                if(!is_null($vinculosGravados)){
                    if(!is_null($message)){
                        $message.= '<br><br>';
                    }
                    $message.= "O(s) Vínculo(s) <br> ".$vinculosGravados." <br> já estava(m) gravado(s).\n";
                }
                if(!is_null($vinculosSemEscala)){
                    if(!is_null($message)){
                        $message.= '<br><br>';
                    }
                    $message.= "O(s) Vínculo(s) <br> ".$vinculosSemEscala." <br> não encontrado(s) escala de trabalho.\n";
                }

                if(!is_null($message)){
                    $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message, null, null, $apurado));
                }
            }
            catch(\Exception $e) {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retrocederApuracao(DTO\Request\DataSet $request, DTO\Response $response) {
       try {            
            set_time_limit(0);
            foreach($request->getDataSet()->getRows() as $row) {
                $nrPeriodoApuracao = $row['NRPERIODOAPURACAO'];
                $iniPeriodo         = str_replace(' 00:00:00', '', $row['DTINICIAL']);
                $fimPeriodo         = str_replace(' 00:00:00', '', $row['DTFINAL']);
                $ordem              = $row['DSORDEM'];
                $dsInicio           = $row['DSINICIO'];
                $dsFim              = $row['DSFIM'];
                $nrTipoVinculo      = $row['NRTIPOVINCULOM'];
                $nrOrg              = $row['NRORGTRAB'];
                $nrOrgPadrao        = $row['NRORGPADRAO'];
                $dtCompetencia      = $row['DTCOMPETENCIA'];
                $apuraRescindidos   = $row['IDAPURARESCINDIDOS'];
                
                $requestWebService = !in_array($nrOrg, self::ORGS_APURACAO_LOCAL);

                switch($ordem){
                    case self::Alfabetica:
                        $criterio = 'NMVINCULOM';
                        break;
                    case self::Data_Admissao:
                        $criterio = 'DTADMISSAOVINC';
                        break;
                    case self::Matricula:
                        $criterio = 'CDMATRICULA';
                        break;
                    case self::Estrutura:
                        $criterio = 'NRESTRUTLEGAL';
                        break;
                    default:
                        $criterio = 'TODOS';
                        break;
                }
            }
            $vinculos = $this->apuracaoFrequenciaService->getVinculosPorSelecaoNRescindidos($criterio, $dsInicio, $dsFim, $dtCompetencia, $nrTipoVinculo, $nrOrgPadrao, $nrOrg, $apuraRescindidos, $fimPeriodo, $iniPeriodo);
            
            // Valida se foram encontrados vínculos
            if (!is_array($vinculos) || count($vinculos) < 1) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM07022_PTO_FILTRO_VINC_NOT_FOUND));
                return;
            }
            
            try{    
                foreach($vinculos as $vinculo) {
                    if ($requestWebService) {
                        $this->apuracaoFrequenciaService->requestRetrocessoApuracaoWebService($vinculo['NRVINCULOM'], $nrPeriodoApuracao);
                    } else {
                        $this->apuracaoFrequenciaService->retrocessoApuracao($nrOrg, $vinculo['NRVINCULOM'], $nrPeriodoApuracao);
                    }
                }
            } catch(\Exception $ex) {
                $response->setError(new \Zeedhi\Framework\DTO\Response\Error($ex->getMessage(), $ex->getCode()));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function simularApuracao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            set_time_limit(0);
            
            foreach($request->getDataSet()->getRows() as $row) {
                $nrPeriodoApuracao = $row['NRPERIODOAPURACAO'];
                $iniPeriodo         = str_replace(' 00:00:00', '', $row['DTINICIAL']);
                $fimPeriodo         = str_replace(' 00:00:00', '', $row['DTFINAL']);
                $ordem              = $row['DSORDEM'];
                $dsInicio           = $row['DSINICIO'];
                $dsFim              = $row['DSFIM'];
                $nrTipoVinculo      = $row['NRTIPOVINCULOM'];
                $nrOrg              = $row['NRORGTRAB'];
                $nrOrgPadrao        = $row['NRORGPADRAO'];
                $dtCompetencia      = $row['DTCOMPETENCIA'];
                $apuraRescindidos   = $row['IDAPURARESCINDIDOS'];

                switch($ordem){
                    case self::Alfabetica:
                        $criterio = 'NMVINCULOM';
                        break;
                    case self::Data_Admissao:
                        $criterio = 'DTADMISSAOVINC';
                        break;
                    case self::Matricula:
                        $criterio = 'CDMATRICULA';
                        break;
                    case self::Estrutura:
                        $criterio = 'NRESTRUTLEGAL';
                        break;
                    default:
                        $criterio = 'TODOS';
                        break;
                }
            }

            //Busca todos os vínculos de acordo com os critérios da seleção
            $vinculos = $this->apuracaoFrequenciaService->getVinculosPorSelecaoNRescindidos($criterio, $dsInicio, $dsFim, $dtCompetencia, $nrTipoVinculo, $nrOrgPadrao, $nrOrg, $apuraRescindidos, $fimPeriodo, $iniPeriodo);
            
            // Valida se foram encontrados vínculos
            if (!is_array($vinculos) || count($vinculos) < 1) {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message(Messages::HCM07022_PTO_FILTRO_VINC_NOT_FOUND));
                return;
            }
            
            $apuracoes = array();
            $apuracoesToFront = array();
            $vinculosApurados = null;
            $vinculosGravados = null;
            $vinculosSemControle = null;
            $vinculosSimulados = array();
            $vinculosNaoSimulados = array();
            
            $vinculosSelecionadosSimulacao = array_slice($vinculos, self::INDEX_INICIO_VINCULOS_SIMULACAO, self::QUANT_VINCULOS_SIMULACAO);

            $apurado = false;
            if($vinculosSelecionadosSimulacao){
                $message = null;
                foreach($vinculosSelecionadosSimulacao as $vinculo){
                    $idnaocontrolajornada =  $this->apuracaoFrequenciaService->verificaControleJornada($nrOrg, $vinculo['NRVINCULOM'], $fimPeriodo);

                    if ($idnaocontrolajornada == 'N') {
                        $verificaResumo = $this->apuracaoFrequenciaService->getResumoSituFreqVinculo($nrOrg, $vinculo['NRVINCULOM'], $nrPeriodoApuracao);

                        if (!is_array($verificaResumo)) {
                            
                            $apuracao = new Apuracao($nrOrg, $vinculo['NRVINCULOM'], $iniPeriodo,
                                                     $fimPeriodo, $nrPeriodoApuracao);
                            $apuracoes[$vinculo['NRVINCULOM']] = $apuracao;
                            $apuracoesToFront[$vinculo['NRVINCULOM']] = $apuracao->toJson();
                            $apurado = true;
                            
                            if(!is_null($vinculosApurados)){
                                 $vinculosApurados.= '<br>';
                            }
                            $vinculosApurados .= $vinculo['NRVINCULOM'];
                            array_push($vinculosSimulados, $vinculo);
                
                      } else {
                            if(!is_null($vinculosGravados)){
                                 $vinculosGravados.= '<br>';
                            }
                            $vinculosGravados.= $vinculo['NMVINCULOM'];
                        }
                    } else {
                        if(!is_null($vinculosSemControle)){
                             $vinculosSemControle.= '<br>';
                        }
                        $vinculosSemControle.= $vinculo['NMVINCULOM'];
                    }
                }
            } else {
                $message = 'Nenhum vinculo encontrado para os parâmetros selecionados';
            }
            if(!is_null($vinculosSemControle)){
                if(!is_null($message)){
                    $message.= '<br><br>';
                }
                $message .= "O(s) Vínculo(s) <br> ".$vinculosSemControle." <br> não possui controle de jornada.\n";
            }
            if(!is_null($vinculosGravados)){
                if(!is_null($message)){
                    $message.= '<br><br>';
                }
                $message .= "O(s) Vínculo(s) <br> ".$vinculosGravados." <br> já estava(m) gravado(s).\n";
            }

            if(!is_null($message)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message($message));
            }
            if ($apurado) {
                $this->hcmParameters->set('apuracoesFrequencia', $apuracoes);
                $dados = array('NRPERIODOAPURACAO' => $nrPeriodoApuracao,
                'DTINICIAL' => $iniPeriodo,
                'DTFINAL' => $fimPeriodo,
                'DSORDEM' => $ordem,
                'DSINICIO' => $dsInicio,
                'DSFIM' => $dsFim,
                'NRTIPOVINCULOM' => $nrTipoVinculo,
                'NRORGTRAB' => $nrOrg,
                'NRORGPADRAO' => $nrOrgPadrao,
                'DTCOMPETENCIA' => $dtCompetencia,
                'IDAPURARESCINDIDOS' => $apuraRescindidos,
                'NRVINCULOM' => $vinculosSelecionadosSimulacao,
                'SIMULADOS' => $vinculosSimulados);
                $this->hcmParameters->set('dadosSimulacao', $dados);

                $response->addDataSet(new DataSet($this->dataSourceSetApuracaoFrequencia, array('type' => 'success',  'NRPERIODOAPURACAO' => $nrPeriodoApuracao,
                'DTINICIAL' => $iniPeriodo,
                'DTFINAL' => $fimPeriodo,
                'DSORDEM' => $ordem,
                'DSINICIO' => $dsInicio,
                'DSFIM' => $dsFim,
                'NRTIPOVINCULOM' => $nrTipoVinculo,
                'NRORGTRAB' => $nrOrg,
                'NRORGPADRAO' => $nrOrgPadrao,
                'DTCOMPETENCIA' => $dtCompetencia,
                'IDAPURARESCINDIDOS' => $apuraRescindidos,
                'NRVINCULOM' => $vinculosApurados)));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function getTotalizacaoSimulacao (DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $parameters = array();
        foreach ($filters as $key => $filter) {
            $parameters[$filter['columnName']] = $filter['value'];
        }
        $apuracoes =  $this->hcmParameters->getApuracoesFrequencia();
        $data      = DateUtil::getDataDeString($parameters['DATA'],
            DateUtil::FORMATO_BRASILEIRO,
            true);
            
        if (!isset($apuracoes[$parameters['NRVINCULOM']])) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRegistroSimulacao', []));
            return;
        }    
        
        $diaPeriodo = $apuracoes[$parameters['NRVINCULOM']]->getDiaPeriodo($data);
        $dia        = $diaPeriodo['DIA'];
        $totalizacao = array();
        /** @var Dia $dia */
        if(is_object($dia)){
            $totalizacao = $dia->getDatasourceTotalizacao();
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getTotalizacaoSimulacao', $totalizacao));
    }

    public function getIntervaloSimulacao (DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $parameters = array();
        foreach ($filters as $key => $filter) {
            $parameters[$filter['columnName']] = $filter['value'];
        }
        $apuracoes =  $this->hcmParameters->getApuracoesFrequencia();
        $data      = DateUtil::getDataDeString($parameters['DATA'],
            DateUtil::FORMATO_BRASILEIRO,
            true);
            
        if (!isset($apuracoes[$parameters['NRVINCULOM']])) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRegistroSimulacao', []));
            return;
        }    
        
        $diaPeriodo = $apuracoes[$parameters['NRVINCULOM']]->getDiaPeriodo($data);
        $dia        = $diaPeriodo['DIA'];
         $intervalos = array();

        /** @var $dia \Folha\Logic\ControlePonto\Dia */
        if(is_object($dia)){
            $intervalos = $dia->getDatasourceIntervalos();
        }

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getIntervaloSimulacao', $intervalos));
    }

    public function getRegistroSimulacao (DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();
        $parameters = array();
        foreach ($filters as $key => $filter) {
            $parameters[$filter['columnName']] = $filter['value'];
        }
        $apuracoes =  $this->hcmParameters->getApuracoesFrequencia();
        $data      = DateUtil::getDataDeString($parameters['DATA'],
            DateUtil::FORMATO_BRASILEIRO,
            true);
            
        if (!isset($apuracoes[$parameters['NRVINCULOM']])) {
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRegistroSimulacao', []));
            return;
        }
        
        $diaPeriodo = $apuracoes[$parameters['NRVINCULOM']]->getDiaPeriodo($data);
        $dia        = $diaPeriodo['DIA'];
        $registros = array();

        /** @var $dia \Folha\Logic\ControlePonto\Dia */
        if(is_object($dia)){
            $registros = $dia->getRegistrosDiaDatasource();
        }

        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('getRegistroSimulacao', $registros));
    }
    
    public function getAcompanhamentoPeriodo (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetAcompanhamentoPeriodo);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}
