<?php
namespace HCM\VAB\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Folha\Entities\VabBeneficiario;
use Folha\Entities\GpeVinculom;
use HCM\Util\Messages;
use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use HCM\Util\MsgCalculo;
use Geral\Util\NovoCodigo;
use HCM\Util\HCMParameters;
use HCM\Util\Valor;

class CalculoBeneficioRepository {

    protected  $connection;
 
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
 
    public function buscaVinculos_Mensal($nrorg, $competencia, $ordem, $dsinicio, $dsfim, $nrestruturam, $nrtpbeneficio, $formulario, $dsinicio1, $dsfim1)
    {
        try {
            $dtmescompetencia = DateUtil::getDataDeString($competencia, DateUtil::FORMATO_BRASILEIRO, true);
            $fimcompetencia = $dtmescompetencia->format('t/m/Y');

            // Variável para o filtro de rescisão. Buscar vínculos rescindidos a partir do dia 16 do mês anterior à competência.
            $dtCompetencAnterior = clone $dtmescompetencia; 
            $dtCompetencAnterior->sub(new \DateInterval('P1M'));
            $dtCompetencAnterior = $dtCompetencAnterior->format('d/m/Y');

            // Variável para o filtro de admissao
            $dtCompetencPosterior = clone $dtmescompetencia;
            $dtCompetencPosterior->add(new \DateInterval('P1M'));
            $dtCompetencPosterior = $dtCompetencPosterior->format('d/m/Y');

            // Filtra somente vínculos que possuam histórico
            $vinculosComHistorico = " AND EXISTS (SELECT 1 FROM GPE_VINCULOH WHERE NRVINCULOM = e.NRVINCULOM AND NRORG = e.NRORG) ";

            
            $condEstrutBenef = ($nrestruturam ? "AND VAB_BENEFICIOH.NRESTRUTURAM = $nrestruturam" : "");
            $condTipoBenef = ($nrtpbeneficio ? "AND VAB_BENEFICIOH.NRTPBENEFICIO = $nrtpbeneficio" : "");
            $cond_beneficiario =
                " AND EXISTS (SELECT * FROM VAB_BENEFICIARIO
                                JOIN VAB_BENEFICIOH
                                  ON VAB_BENEFICIOH.NRORG = VAB_BENEFICIARIO.NRORG
                                 AND VAB_BENEFICIOH.NRBENEFICIOM = VAB_BENEFICIARIO.NRBENEFICIOM
                                 $condEstrutBenef
                                 $condTipoBenef
                               WHERE VAB_BENEFICIARIO.NRORG = e.NRORG
                                 AND VAB_BENEFICIARIO.NRVINCULOM = e.NRVINCULOM)";

            switch($ordem)
            {
                case 'ALFABETICA':
                {
                    $param = array(':P_INICIO' => $dsinicio,
                        ':P_FIM' => $dsfim,
                        ':P_NRORG' => $nrorg);

                    //Nome Inicial
                    $whereinivinculo = "NRORG = :P_NRORG AND NRVINCULOM = :P_INICIO"; /** @var GpeVinculom $objinivinculo */
                    $objinivinculo = GpeVinculom::getObjectByWhere($whereinivinculo, $param);

                    //Nome Final
                    $wherefimvinculo = "NRORG = :P_NRORG AND NRVINCULOM = :P_FIM"; /** @var GpeVinculom $objfimvinculo */
                    $objfimvinculo = GpeVinculom::getObjectByWhere($wherefimvinculo, $param);

                    $nomeInicio = $objinivinculo->getNmvinculom();
                    $nomeFinal = $objfimvinculo->getNmvinculom();
                    if($nomeFinal<$nomeInicio){
                        $aux = $nomeFinal;
                        $nomeFinal = $nomeInicio;
                        $nomeInicio = $aux;
                    }

                    $param = array( 
                        ':P_INICIO' => $nomeInicio,
                        ':P_FIM' => $nomeFinal,
                        ':P_NRORG' => $nrorg
                    );
                    $where = "(DTRESCISAOVINC >= '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC < '$dtCompetencPosterior' AND NMVINCULOM BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_NOMES_ERROR;
                    break;
                }
                case 'MATRICULA':{
                    $param = array(
                        'P_INICIO' => $dsinicio,
                        'P_FIM' => $dsfim,
                        'P_NRORG' => $nrorg);
                    $where = "(DTRESCISAOVINC >= '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC < '$dtCompetencPosterior' AND CDMATRICULA BETWEEN :P_INICIO AND :P_FIM AND NRORG = :P_NRORG";
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_MATRICS_ERROR;
                    break;
                }
                case 'DATA_ADMISSAO':{
                    $param = array(':P_NRORG' => $nrorg);
                    $where = "(DTRESCISAOVINC >= '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC BETWEEN '$dsinicio' AND '$dsfim' AND NRORG = :P_NRORG";
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_DATAS_ERROR;
                    break;
                }
                case 'ESTRUTURA':{
                    $param = array( 'P_NRORG' => $nrorg,
                        'P_NRESTRUTLEGAL' =>$dsinicio,
                        'P_NRESTRUTGEREN' =>$dsfim);
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC < '$dtCompetencPosterior' AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND (NRESTRUTGEREN = :P_NRESTRUTGEREN OR :P_NRESTRUTGEREN IS NULL) AND NRESTRUTLEGAL = :P_NRESTRUTLEGAL AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_ESTRUTS_ERROR;
                    break;
                }
                case 'LISTA_VINCULO':
                    $param = array('P_NRORG' => $nrorg);
                    $nrVincList = implode(",", $formulario['NRVINCULOM_LIST']);
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtCompetencPosterior' AND NRVINCULOM IN ($nrVincList)";
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_LISTA_ERROR;
                    break;
                case 'LISTA_LOTE':
                    $param = array('P_NRORG' => $nrorg);
                    $listaLote = implode(",", $formulario['NRLOTEVINC_LIST']);
                    
                    $where = "NRORG = :P_NRORG AND (DTRESCISAOVINC >= '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND DTADMISSAOVINC <= '$dtCompetencPosterior' AND NRVINCULOM IN (SELECT NRVINCULOM FROM FPA_ITEMLOTEVINC WHERE NRORG = :P_NRORG AND NRLOTEVINC IN($listaLote))";
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_LISTA_ERROR;
                    break;
                case 'LISTA_ESTRUTURA':
                    $listaEstrutLegal = isset($formulario['LIST_NRESTRUTLEGAL']) && is_array($formulario['LIST_NRESTRUTLEGAL']) && count($formulario['LIST_NRESTRUTLEGAL']) ? implode(",", $formulario['LIST_NRESTRUTLEGAL']) : null;
                    $listaEstrutGeren = isset($formulario['LIST_NRESTRUTGEREN']) && is_array($formulario['LIST_NRESTRUTGEREN']) && count($formulario['LIST_NRESTRUTGEREN']) ? implode(",", $formulario['LIST_NRESTRUTGEREN']) : 1;

                    $param = array( 'P_NRORG' => $nrorg);
                    
                    if($dsinicio1 && $dsfim1){
                        $where = " DTADMISSAOVINC BETWEEN '$dsinicio1' AND '$dsfim1' AND ";
                    } else if ($dsinicio1){
                        $where = " DTADMISSAOVINC >= '$dsinicio1' AND ";
                    } else if ($dsfim1){
                        $where = " DTADMISSAOVINC <= '$dsfim1' AND ";
                    } else{
                        $where = " DTADMISSAOVINC <= '$dtCompetencPosterior' AND ";
                    }
                                    
                    $where .= "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren) OR  1 IN($listaEstrutGeren)  ) AND NRESTRUTLEGAL IN($listaEstrutLegal) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG) ) ";
                    
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_LISTA_ERROR;
                    break;
                case 'LISTA_NRESTRUTSIND':
                    $listaEstrutLegal = isset($formulario['LIST_NRESTRUTLEGAL']) && is_array($formulario['LIST_NRESTRUTLEGAL']) && count($formulario['LIST_NRESTRUTLEGAL']) ? implode(",", $formulario['LIST_NRESTRUTLEGAL']) : null;
                    $listaEstrutSind = isset($formulario['LIST_NRESTRUTSIND']) && is_array($formulario['LIST_NRESTRUTSIND']) && count($formulario['LIST_NRESTRUTSIND']) ? implode(",", $formulario['LIST_NRESTRUTSIND']) : 1;

                    $param = array( 'P_NRORG' => $nrorg);
                    
                    if($dsinicio1 && $dsfim1){
                        $where = " DTADMISSAOVINC BETWEEN '$dsinicio1' AND '$dsfim1' AND ";
                    } else if ($dsinicio1){
                        $where = " DTADMISSAOVINC >= '$dsinicio1' AND ";
                    } else if ($dsfim1){
                        $where = " DTADMISSAOVINC <= '$dsfim1' AND ";
                    } else{
                        $where = " DTADMISSAOVINC <= '$dtCompetencPosterior' AND ";
                    }
                         
                    $where .= " NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTSIND IN($listaEstrutSind) ) AND NRESTRUTLEGAL IN ($listaEstrutLegal) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG))";
                    
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_LISTA_ERROR;
                    break;
                case 'LISTA_BENEFICIO':
                    $listaBeneficio = isset($formulario['LIST_NRBENEFICIOM']) && is_array($formulario['LIST_NRBENEFICIOM']) && count($formulario['LIST_NRBENEFICIOM']) ? implode(",", $formulario['LIST_NRBENEFICIOM']) : null;

                    $param = array(
                        'P_NRORG' => $nrorg,
                        'P_DTCOMPETENCANTE' => $dtCompetencAnterior,
                        'P_DTCOMPETENCPOST' => $dtCompetencPosterior,
                    );
                                    
                    $where = "NRORG = :P_NRORG 
                        AND (DTRESCISAOVINC > :P_DTCOMPETENCANTE OR DTRESCISAOVINC IS NULL) 
                        AND DTADMISSAOVINC <= :P_DTCOMPETENCPOST 
                        AND NRVINCULOM IN ( SELECT NRVINCULOM FROM VAB_BENEFICIARIO
                                             WHERE NRORG = :P_NRORG
                                               AND NRBENEFICIOM IN ($listaBeneficio)
                                               AND IDATIVO = 'S'
                                               AND DTINSCBENEFICIARIO < :P_DTCOMPETENCPOST
                                               AND (DTCANCBENEFICIARIO IS NULL OR DTCANCBENEFICIARIO >= :P_DTCOMPETENCANTE) )";
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_LISTA_ERROR;
                    break;
                case 'LISTA_TOMADOR':
                    $listaEstrutSup = isset($formulario['LIST_NRESTRUTURAMSUP']) && is_array($formulario['LIST_NRESTRUTURAMSUP']) && count($formulario['LIST_NRESTRUTURAMSUP']) ? implode(",", $formulario['LIST_NRESTRUTURAMSUP']) : null;
                    $listaEstrutGeren = isset($formulario['LIST_NRESTRUTGEREN']) && is_array($formulario['LIST_NRESTRUTGEREN']) && count($formulario['LIST_NRESTRUTGEREN']) ? implode(",", $formulario['LIST_NRESTRUTGEREN']) : 1;
                    
                    $param = array( 'P_NRORG' => $nrorg);

                    if($dsinicio1 && $dsfim1){
                        $where = " DTADMISSAOVINC BETWEEN '$dsinicio1' AND '$dsfim1' AND ";
                    } else if ($dsinicio1){
                        $where = " DTADMISSAOVINC >= '$dsinicio1' AND ";
                    } else if ($dsfim1){
                        $where = " DTADMISSAOVINC <= '$dsfim1' AND ";
                    } else{
                        $where = " DTADMISSAOVINC <= '$dtCompetencPosterior' AND ";
                    }
                    
                    if(isset($listaEstrutSup) && (isset($listaEstrutGeren) && $listaEstrutGeren != 1)){         
                        $where .= "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND ( NRESTRUTGEREN IN($listaEstrutGeren)) AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) ) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG))))";
                    } else {
                        $where .= "NRORG = :P_NRORG AND (DTRESCISAOVINC > '$dtCompetencAnterior' OR DTRESCISAOVINC IS NULL) AND NRVINCULOM IN ( SELECT NRVINCULOM FROM GPE_VINCULOH WHERE NRORG = :P_NRORG AND DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC)DTMESCOMPETENC FROM GPE_VINCULOH MAX_VINCULOH WHERE MAX_VINCULOH.NRVINCULOM = GPE_VINCULOH.NRVINCULOM AND MAX_VINCULOH.NRORG = GPE_VINCULOH.NRORG) AND EXISTS (SELECT 1 FROM ESTRUTURAM EMA WHERE EMA.NRORG = :P_NRORG AND EMA.NRESTRUTURAM = (SELECT H.NRESTRUTURAMSUP FROM HIERARQUIAORG H WHERE H.NRORG = :P_NRORG AND H.NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND H.NRESTRUTURAMSUP IN ($listaEstrutSup) AND H.NRTIPOHIERARQORG = (SELECT NRTIPOHIERARQORG FROM TIPOHIERARQORG WHERE NRORG IN (0, :P_NRORG) AND NRTIPOESTRUTSUP = (SELECT NVL(NRTPESTRUTOMADOR, 7) FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) AND NRTIPOESTRUTREL = (SELECT NRTPESTRUTGEREN FROM GPE_PARAMORG WHERE NRORG = :P_NRORG) ) AND H.DTINIVIGENCIA = (SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG WHERE DTINIVIGENCIA <= '$fimcompetencia' AND NRORG = :P_NRORG AND NRESTRUTURAMREL = GPE_VINCULOH.NRESTRUTGEREN AND NRTIPOHIERARQORG = H.NRTIPOHIERARQORG))))";
                    }
                    
                    $msgerro = Messages::HCM00000_GER_VINCS_N_ENCONT_LISTA_ERROR;
                    break;
                default: {
                return null;
                }
            }
            $where .= $vinculosComHistorico . $cond_beneficiario;
            
            $result = GpeVinculom::getObjectsByWhere($where, $param);
            
            if ($result) {
                return $result;
            } else {
                if ($condEstrutBenef || $condTipoBenef) {
                    throw new \Exception(Messages::HCM00000_GER_VINC_N_ENCONTR_FILTRO_INF_ERROR, Messages::ERROR_MESSAGE_CODE);
                } else {
                    throw new \Exception($msgerro, Messages::ERROR_MESSAGE_CODE);
                }
            }

        } catch (\Exception $e) {
            throw $e;
        }
    }
    

    public function getBeneficiarioPorVinculo($nrorg, $nrvinculom, $dtmescompetenc, $tpperiodo, $nrtpbeneficio=null, $nrestruturam=null){
        $condTpbeneficio = $nrtpbeneficio ? "AND BH.NRTPBENEFICIO = $nrtpbeneficio" : "";
        $condEstrutura = $nrestruturam ? "AND BH.NRESTRUTURAM = $nrestruturam" : "";
        $where =
            "NRVINCULOM = $nrvinculom AND
            NRORG = $nrorg AND
            NRBENEFICIOM IN (SELECT BM.NRBENEFICIOM
                               FROM VAB_BENEFICIOM BM
                               JOIN VAB_BENEFICIOH BH ON BH.NRORG = BM.NRORG
                                                     AND BH.NRBENEFICIOM = BM.NRBENEFICIOM
                                                     AND BH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM VAB_BENEFICIOH
                                                                               WHERE NRORG = BH.NRORG
                                                                                 AND NRBENEFICIOM = BH.NRBENEFICIOM
                                                                                 AND DTMESCOMPETENC <= '$dtmescompetenc')
                              WHERE BM.NRORG = $nrorg
                                AND BM.IDATIVO = 'S'
                                AND BH.TPPERIODOBENEF = '$tpperiodo'
                                $condTpbeneficio
                                $condEstrutura)";

        $beneficiarios = VabBeneficiario::getObjectsByWhere($where, array(), 'NRBENEFICIOM ASC, NREVENTOM ASC');
        return $beneficiarios;
    }
    
    public function getBeneficioPorVinculo($dtmescompetenc, $nrorg, $nrbeneficiom){
        $query = <<<SQL
    SELECT
        BNM.*,
        BNH.*
    FROM
        VAB_BENEFICIOM BNM 
    INNER JOIN
        VAB_BENEFICIOH BNH 
            ON BNH.NRBENEFICIOM = BNM.NRBENEFICIOM 
            AND(
                BNM.NRORG = BNH.NRORG 
            ) 
            AND BNH.DTMESCOMPETENC =(
                    SELECT
                        MAX(BNEH.DTMESCOMPETENC) 
                FROM
                    VAB_BENEFICIOH BNEH 
                WHERE
                    BNEH.NRORG = BNH.NRORG 
                    AND BNEH.NRBENEFICIOM = BNH.NRBENEFICIOM 
                    AND BNEH.DTMESCOMPETENC <= :DTMESCOMPETENC
            )        
    WHERE 
        BNM.NRBENEFICIOM = :NRBENEFICIOM
        AND BNM.NRORG = :NRORG
SQL;

        $retorno = $this->connection->fetchAll($query,[ 
                                                    "DTMESCOMPETENC" => $dtmescompetenc,
                                                    "NRORG"         => $nrorg,
                                                    "NRBENEFICIOM"  => $nrbeneficiom,
                                                   ]
                                           );
        return $retorno[0];
    }
    
    public function getBeneficioDescontoSind($dtmescompetenc, $nrorg, $nrTpBeneficio, $nrEstrutSind){
        $query = <<<SQL
    SELECT
        BCCT.*
    
    FROM 
        FPA_CADASTROCCT CCT 

    JOIN
        VAB_BENEFICIOCCT BCCT
            ON BCCT.NRCADASTROCCT = CCT.NRCADASTROCCT
            AND BCCT.NRORG = CCT.NRORG
    
    WHERE 
        BCCT.NRTPBENEFICIO = :NRTPBENEFICIO
        AND CCT.NRESTRUTURAM = :NRESTRUTURASIND
        AND CCT.NRORG = :NRORG
        AND CCT.IDATIVO = 'S'
        AND CCT.DTANOBASECCT = (SELECT MAX(DTANOBASECCT) FROM FPA_CADASTROCCT CCT2
                        WHERE CCT2.IDATIVO = 'S'
                        AND CCT2.NRORG = CCT.NRORG
                        AND CCT2.NRESTRUTURAM = CCT.NRESTRUTURAM
                        AND CCT2.DTANOBASECCT <= :DTMESCOMPETENC
                       )
SQL;
        $params = [ 
                    "DTMESCOMPETENC"  => $dtmescompetenc,
                    "NRORG"           => $nrorg,
                    "NRTPBENEFICIO"   => intval($nrTpBeneficio),
                    "NRESTRUTURASIND" => intval($nrEstrutSind),
                  ];
        
        $retorno = $this->connection->fetchAll($query, $params);
        
        if(is_array($retorno) && isset($retorno[0])) $retorno = $retorno[0];
        else $retorno = null;

        return $retorno;
    }
    
    public function retrocederCalculos_Mensal($nrcalculobenef, $nrvinculom, $dtmescompetenc, $nrorg, $eventos, $dtpagamento){
        
        $qtdeDeletados = 0;
        
        $IN = " AND NREVENTOM IN (";

        foreach($eventos as $key=>$value){
            $row = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy( 
                [ 
                 'nrcalculofolha' => $nrcalculobenef,
                 'nrvinculom'     => $nrvinculom,
                 'nreventom'      => $value,
                 'nrorg'          => $nrorg,
                 'dtocorrencia'   => Dateutil::getDataDeString($dtpagamento, DateUtil::FORMATO_BRASILEIRO, true),
                ]
            );
            
            if($row){
                if($row->getNrlotelancevento())
                    MsgCalculo::getInstance()->adicionaMensagem('Evento '.$value.' para o vínculo '.$nrvinculom.' já exportado para folha. ', MsgCalculo::ALERTA);
                else{
                    $IN .= $value.', ';
                    $qtdeDeletados++;
                }    
            }        

        }

        $virgula = strripos($IN, ',');
        
        if($virgula){ 
            $IN = substr($IN, 0, -2);
            $IN .= ")";
        }else
            $IN .= 'null)';
        
        
        // Retrocesso na tabela de itens de cálculo
        $query = $this->entityManager->getConnection()->prepare("
            DELETE 
            FROM FPA_ITECALCFOLHA
            WHERE 
                NRORG = :NRORG
                AND NRVINCULOM = :NRVINCULOM 
                AND DTOCORRENCIA = :DTOCORRENCIA
                AND NRCALCULOFOLHA = :NRCALCULOFOLHA".$IN
        );

        $query->execute(
            [
                "NRVINCULOM"     => $nrvinculom,
                "DTOCORRENCIA"   => $dtpagamento,
                "NRORG"          => $nrorg,
                "NRCALCULOFOLHA" => $nrcalculobenef,
            ]
        );
        
        // Retrocesso na tabela VAB_APURABENEFICIO
        $query = $this->entityManager->getConnection()->prepare("
            DELETE 
            FROM VAB_APURABENEFICIO
            WHERE 
                NRORG = :NRORG
                AND NRVINCULOM = :NRVINCULOM 
                AND NRCALCULOFOLHA = :NRCALCULOFOLHA".$IN
        );

        $query->execute(
            [
                "NRVINCULOM"     => $nrvinculom,
                "NRORG"          => $nrorg,
                "NRCALCULOFOLHA" => $nrcalculobenef,
            ]
        );


        return $qtdeDeletados;

    }

    public function retrocederCalculos_Diario($nrcalculobenef, $nrvinculom, $dtmescompetenc, $nrorg, $eventos, $dtpagamento){
        
        $qtdeDeletados = 0;
        
        $IN = " AND NREVENTOM IN (";

        foreach($eventos as $key=>$value){
            
            $row = $this->entityManager->getRepository(Repositories::VAB_HISTBENEFVINC)->findOneBy( 
                [ 
                 'nrcalculofolha' => $nrcalculobenef,
                 'dtmescompetenc' => Dateutil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true),
                 'nrvinculom'     => $nrvinculom,
                 'nreventom'      => $value,
                 'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
                 'dtpgtobenef'    => Dateutil::getDataDeString($dtpagamento, DateUtil::FORMATO_BRASILEIRO, true)
                ]
            );

            if($row){
                $IN .= $value.', ';
                $qtdeDeletados++;
            }        
        }

        $virgula = strripos($IN, ',');
        
        if($virgula){ 
            $IN = substr($IN, 0, -2);
            $IN .= ")";
        }else
            $IN .= 'null)';
        
        
        // Retrocesso na tabela de VAB_HISTBENEFVINC
        $query = $this->entityManager->getConnection()->prepare("
            DELETE 
            FROM VAB_HISTBENEFVINC
            WHERE 
                NRORG = :NRORG
                AND NRVINCULOM = :NRVINCULOM 
                AND DTMESCOMPETENC = :DTCOMPETENC
                AND DTPGTOBENEF = :DTOCORRENCIA
                AND NRCALCULOFOLHA = :NRCALCULOFOLHA".$IN
                
        );

        $query->execute(
            [
                "NRVINCULOM"     => $nrvinculom,
                "DTOCORRENCIA"   => $dtpagamento,
                "DTCOMPETENC"   => $dtmescompetenc,
                "NRORG"          => $nrorg,
                "NRCALCULOFOLHA" => $nrcalculobenef,
            ]
        );

        return $qtdeDeletados;

    }
    
    public function persistirCalculoDIRF_Mensal($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrPessoaBeneficiario, $proporcao, $nrEventoValorEvento, $tipoEvento){
        
        $row_fpaIteCalcFolha = null;
        
        $row_fpaIteCalcFolha = $this->entityManager->getRepository(Repositories::FPA_ITECALCFOLHA)->findOneBy( 
            [ 
             'nrcalculofolha' => $nrcalculobenef,
             'nrvinculom'     => $nrvinculom,
             'nreventom'      => key($nrEventoValorEvento),
             'nrorg'          => $this->hcmParameters->getNrorgTrabalho(),
             'dtocorrencia'   => Dateutil::getDataDeString($this->hcmParameters->getDtCompetencia(), DateUtil::FORMATO_BRASILEIRO, true),
            ]
        );


        if (
                ( !$row_fpaIteCalcFolha || !$row_fpaIteCalcFolha->getNrlotelancevento() ) &&
                count($nrEventoValorEvento) > 0 && ($nrEventoValorEvento[key($nrEventoValorEvento)] >= 0) 
            
           ) {
               
            $this->deleteVabApuraBeneficio($nrcalculobenef, $nrvinculom, key($nrEventoValorEvento), $nrPessoaBeneficiario, $tipoEvento, $nrbeneficiom);
            
            $vlrEvento = $nrEventoValorEvento[key($nrEventoValorEvento)];

            $eventoh = $this->entityManager->getRepository(Repositories::FPA_EVENTOM)->retornaEvento(
                key($nrEventoValorEvento),
                $this->hcmParameters->getNrorgTrabalho(), 
                $this->hcmParameters->getNrorgPadrao(), 
                $this->hcmParameters->getDtCompetencia() 
            );                    

             $vlrEvento = Valor::arredonda(
                $vlrEvento,
                $eventoh->historico->getIdtipoarredonda(), 
                $eventoh->historico->getNrdigarredonda(),
                $eventoh->historico->getVrlimitearred()
            );

            $query = $this->entityManager->getConnection()->prepare("
                INSERT INTO VAB_APURABENEFICIO (          
                  NRAPURABENEFICIO,
                  NRCALCULOFOLHA,
                  NRVINCULOM,
                  NRPESSOA,
                  NRBENEFICIOM,
                  DSTIPOBENEFICIO,
                  NREVENTOM,
                  VRAPURABENEFICIO,
                  NRORG,
                  IDATIVO,
                  DTINCLUSAO,
                  DTULTATU,
                  NRORGINCLUSAO,
                  NRORGULTATU,
                  CDOPERINCLUSAO,
                  CDOPERULTATU              
                ) 
                
                VALUES (
                  :P_NRAPURABENEFICIO,
                  :P_NRCALCULOFOLHA,
                  :P_NRVINCULOM,
                  :P_NRPESSOA,
                  :P_NRBENEFICIOM,
                  :P_DSTIPOBENEFICIO,
                  :P_NREVENTOM,
                  :P_VRAPURABENEFICIO,
                  :P_NRORG,
                  :P_IDATIVO,
                  :P_DTINCLUSAO,
                  :P_DTULTATU,
                  :P_NRORGINCLUSAO,
                  :P_NRORGULTATU,
                  :P_CDOPERINCLUSAO,
                  :P_CDOPERULTATU              
                )"
            );
            $query->execute(
                array(
                    "P_NRAPURABENEFICIO" => NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_APURABENEFICIO', 1, 22, true),
                    "P_NRCALCULOFOLHA"   => $nrcalculobenef,
                    "P_NRVINCULOM"       => $nrvinculom,
                    "P_NRPESSOA"         => $nrPessoaBeneficiario,
                    "P_NRBENEFICIOM"     => $nrbeneficiom,
                    "P_DSTIPOBENEFICIO"  => $tipoEvento,
                    "P_NREVENTOM"        => key($nrEventoValorEvento),
                    "P_VRAPURABENEFICIO" => $vlrEvento,
                    "P_NRORG"            => $this->hcmParameters->getNrorgTrabalho(),
                    "P_IDATIVO"          => "S",
                    "P_DTINCLUSAO"       => DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "P_DTULTATU"         => DateUtil::getDataAtual()->format('d/m/Y h:m:s'),
                    "P_NRORGINCLUSAO"    => $this->hcmParameters->getNrorgTrabalho(),
                    "P_NRORGULTATU"      => $this->hcmParameters->getNrorgTrabalho(),
                    "P_CDOPERINCLUSAO"   => $this->hcmParameters->getCdoperador(),
                    "P_CDOPERULTATU"     => $this->hcmParameters->getCdoperador()
                )
            );       

        }

    }
    
    public function deleteVabApuraBeneficio($nrcalculobenef, $nrvinculom, $nrEvento, $nrPessoaBeneficiario, $tipoEvento, $nrbeneficiom){
        $query = $this->entityManager->getConnection()->prepare("
            DELETE FROM VAB_APURABENEFICIO
            WHERE 
                NRORG = :P_NRORG 
                AND NRCALCULOFOLHA = :P_NRCALCULOFOLHA 
                AND NRVINCULOM = :P_NRVINCULOM 
                AND NRPESSOA = :P_NRPESSOA
                AND NREVENTOM = :P_NREVENTOM
                AND DSTIPOBENEFICIO = :P_DSTIPOBENEFICIO
                AND NRBENEFICIOM = :P_NRBENEFICIOM"
        );
        $query->execute(
            array(
                "P_NRCALCULOFOLHA"   => $nrcalculobenef,
                "P_NRVINCULOM"       => $nrvinculom,
                "P_NREVENTOM"        => $nrEvento,
                "P_DSTIPOBENEFICIO"  => $tipoEvento,
                "P_NRORG"            => $this->hcmParameters->getNrorgTrabalho(),
                "P_NRPESSOA"         => $nrPessoaBeneficiario,
                "P_NRBENEFICIOM"     => $nrbeneficiom
            )
        );          
    }
    
    public function persistirHistorico_Diario($nrcalculobenef, $nrbeneficiom, $nrvinculom, $nrEvento, $valorEvento, $arrayDiasCalculados, $dtIni, $dtFim, $dtpagamento, $vlrTotalBenef, $qtdeBeneficio){
            
            $query = $this->entityManager->getConnection()->prepare("
                INSERT INTO VAB_HISTBENEFVINC 
                (
                    NRHISTBENEFVINC,
                    NRBENEFICIOM,
                    NRVINCULOM,
                    NRCALCULOFOLHA,
                    NREVENTOM,
                    NRORG,
                    DTMESCOMPETENC,
                    DTINIAPURBENEF,
                    DTFINAPURBENEF,
                    DTPGTOBENEF,
                    QTAPURBENEF,
                    QTDIASBENEF,
                    QTDIASAFASTA,
                    QTDIASFALTAS,
                    QTDIASFERIAS,
                    QTDIASAVISO,
                    QTDIASDSR,
                    VRBENEFICIO,
                    VRDEDUCAOBENEF,
                    VRTOTALBENEF,
                    IDATIVO,
                    NRORGINCLUSAO,
                    DTINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    DTULTATU,
                    CDOPERULTATU,
                    QTDIASFERIADO,
                    QTDIASTRABPERIODO,           
                    QTDIASATESTADO
                )

                VALUES (
                    :P_NRHISTBENEFVINC,
                    :P_NRBENEFICIOM,
                    :P_NRVINCULOM,
                    :P_NRCALCULOFOLHA,
                    :P_NREVENTOM,
                    :P_NRORG,
                    :P_DTMESCOMPETENC,
                    :P_DTINIAPURBENEF,
                    :P_DTFINAPURBENEF,
                    :P_DTPGTOBENEF,
                    :P_QTAPURBENEF,
                    :P_QTDIASBENEF,
                    :P_QTDIASAFASTA,
                    :P_QTDIASFALTAS,
                    :P_QTDIASFERIAS,
                    :P_QTDIASAVISO,
                    :P_QTDIASDSR,
                    :P_VRBENEFICIO,
                    :P_VRDEDUCAOBENEF,
                    :P_VRTOTALBENEF,
                    :P_IDATIVO,
                    :P_NRORGINCLUSAO,
                    :P_DTINCLUSAO,
                    :P_CDOPERINCLUSAO,
                    :P_NRORGULTATU,
                    :P_DTULTATU,
                    :P_CDOPERULTATU,
                    :P_QTDIASFERIADO,
                    :P_QTDIASTRABPERIODO,           
                    :P_QTDIASATESTADO           
                )"
            );

            $query->execute(
                array(
                    "P_NRHISTBENEFVINC"   => intval(NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_HISTBENEFVINC', 1, 22, true)),
                    "P_NRBENEFICIOM"      => intval($nrbeneficiom),
                    "P_NRVINCULOM"        => intval($nrvinculom),
                    "P_NRCALCULOFOLHA"    => intval($nrcalculobenef),
                    "P_NREVENTOM"         => intval($nrEvento),
                    "P_NRORG"             => intval($this->hcmParameters->getNrorgTrabalho()),
                    
                    "P_DTMESCOMPETENC"    => $this->hcmParameters->getDtCompetencia(),
                    "P_DTINIAPURBENEF"    => substr($dtIni, 0, 10),
                    "P_DTFINAPURBENEF"    => substr($dtFim, 0, 10),
                    "P_DTPGTOBENEF"       => substr($dtpagamento, 0, 10),
                    
                    "P_QTAPURBENEF"       => intval($qtdeBeneficio),
                    "P_QTDIASBENEF"       => intval($arrayDiasCalculados['DIASCONSIDERACAO']),
                    "P_QTDIASAFASTA"      => intval($arrayDiasCalculados['DIASAFASTAMENTO']),
                    "P_QTDIASFALTAS"      => intval($arrayDiasCalculados['DIASFALTAS']),
                    "P_QTDIASFERIAS"      => intval($arrayDiasCalculados['DIASFERIAS']),
                    "P_QTDIASAVISO"       => intval($arrayDiasCalculados['DIASAVISO']),
                    "P_QTDIASDSR"         => intval($arrayDiasCalculados['DIASDSR']),
                    "P_QTDIASATESTADO"    => intval($arrayDiasCalculados['DIASATESTADO']),
                    "P_VRBENEFICIO"       => $valorEvento,
                    "P_VRDEDUCAOBENEF"    => intval(0),
                    "P_VRTOTALBENEF"      => $vlrTotalBenef,
                    "P_IDATIVO"           => "S",
                    "P_NRORGINCLUSAO"     => intval($this->hcmParameters->getNrorgTrabalho()),
                    "P_DTINCLUSAO"        => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                    "P_CDOPERINCLUSAO"    => intval($this->hcmParameters->getCdoperador()),
                    "P_NRORGULTATU"       => intval($this->hcmParameters->getNrorgTrabalho()),
                    "P_DTULTATU"          => DateUtil::getDataAtual()->format('d/m/Y H:i:s'),
                    "P_CDOPERULTATU"      => intval($this->hcmParameters->getCdoperador()),
                    "P_QTDIASFERIADO"     => intval($arrayDiasCalculados['DIASFERIADO']),
                    "P_QTDIASTRABPERIODO" => intval($arrayDiasCalculados['DIASDETRABALHONOPERIODO']),
                )
            );

    }

}