/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/******************************************************************************/
/*                                                                            */
/* Nome: TRG_INTERFACE_TECNISA                                                */
/*                                                                            */
/* Descrio:                                                                 */
/* Gera lanamento de estoque no Sistema Tecfood a partir de lanamentos de   */
/* entrada no sistema hospitalar                                              */
/*                                                                            */
/* Criao:                                                                   */
/* 05/02/2004 - Roberto Martins                                               */
/*                                                                            */
/******************************************************************************/

create or replace trigger TRG_INTERFACE_TECNISA
AFTER DELETE OR INSERT OR UPDATE
ON HACOMP.MOVIMENTOS_RECEBIMENTOS FOR EACH ROW
DECLARE
   V_TIES_CODIGO  NUMBER;
   V_MATR_CODIGO  NUMBER;
   V_CDFILIAL     VARCHAR2(04);
   V_NRLANCTONF   VARCHAR2(20);
   V_NRLANCESTQ   VARCHAR2(20);
   V_CDEMPRESA    VARCHAR2(02);
   V_CDINSCESTA   VARCHAR2(16);
   V_CDINSCMUNI   VARCHAR2(16);

   V_NRSEQITPROD  VARCHAR2(04);
   V_CDPRODUTO    VARCHAR2(10);
   V_CDPRODESTO   VARCHAR2(10);
   V_NMPRODUTO    VARCHAR2(50);
   V_VRFATOCONV   NUMBER;
   V_VRTOTAITEM   NUMBER;
   V_VRCONTITEM   NUMBER;

   V_DTINICIAL    DATE;
   V_DTFINAL      DATE;
   V_DTPROCESSA   DATE;
BEGIN

   SELECT NVL(MAX(DTPROCESSA),SYSDATE)
   INTO  V_DTPROCESSA
   FROM TECNISA.PARAMFILIAL
   WHERE CDFILIAL = '0001';

   /*Verificao se o produto  um alimento*/
   BEGIN
       SELECT MATR_CODIGO, TIES_CODIGO
       INTO V_MATR_CODIGO, V_TIES_CODIGO
       FROM HAESTQ.MATERIAIS_EMPRESAS ME
   WHERE
     SEQUENCIA     = NVL(:NEW.MAEM_SEQUENCIA,  :OLD.MAEM_SEQUENCIA);
   EXCEPTION
      WHEN NO_DATA_FOUND THEN
         V_MATR_CODIGO := NULL
         V_TIES_CODIGO := NULL;
   END;

   IF V_TIES_CODIGO = 2 THEN
      /*Somente realiza a atualizao se produto possuir cdigo de integrao*/
      SELECT MAX(CDPRODUTO), MAX(CDPRODESTO),
      MAX(NMPRODUTO),      MAX(VRFATOCONV)
      INTO V_CDPRODUTO, V_CDPRODESTO, V_NMPRODUTO, V_VRFATOCONV
      FROM TECNISA.PRODUTO
      WHERE TO_NUMBER(NVL(CDPRODINTE,'0')) = V_MATR_CODIGO;


      IF ((UPDATING) OR (INSERTING)) AND (V_CDPRODUTO IS NOT NULL) THEN
         IF UPDATING THEN
            /*Excluso na tabela de itens da nota fiscal*/
            DELETE FROM TECNISA.ITEMPRODUTO
            WHERE USR_SEQUENCIA      = :NEW.SEQUENCIA AND
                  USR_RECB_SEQUENCIA = :NEW.RECB_SEQUENCIA;

            /*Excluso na tabela de itens de estoque*/
            DELETE FROM TECNISA.ITLANCTOEST
            WHERE USR_SEQUENCIA      = :NEW.SEQUENCIA AND
                  USR_RECB_SEQUENCIA = :NEW.RECB_SEQUENCIA;
         END IF;

         IF (:NEW.QT_VALIDADA <> 0) AND (:NEW.ST_RECEBIMENTO = 'N') THEN
            /*Verificao se o corpo do lanamento j existe no banco*/
            SELECT MAX(CDFILIAL), MAX(NRLANCTONF), MAX(NRLANCESTQ)
            INTO V_CDFILIAL, V_NRLANCTONF, V_NRLANCESTQ
            FROM TECNISA.LANCTOESTOQ
            WHERE USR_RECB_SEQUENCIA = :NEW.RECB_SEQUENCIA;

            IF V_CDFILIAL IS NULL THEN
               /*Sequencial para chave das tabelas de lanamento*/
               /*Tabela Nota Fiscal*/
               SELECT LPAD(TO_CHAR(TO_NUMBER(NVL(MAX(NRSEQUENCIAL),'0')) + 1),10,'0')
               INTO V_NRLANCTONF
               FROM TECNISA.NOVOCODIGO
               WHERE CDCONTADOR = 'LANCNF/FILIAL' || '0001';

               IF V_NRLANCTONF = '0000000001' THEN
                  INSERT INTO TECNISA.NOVOCODIGO
                     (CDCONTADOR, NRSEQUENCIAL)
                  VALUES
                     ('LANCNF/FILIAL' || '0001', LPAD(V_NRLANCTONF,20,'0'));
               ELSE
                  UPDATE TECNISA.NOVOCODIGO
                  SET NRSEQUENCIAL = LPAD(V_NRLANCTONF,20,'0')
                  WHERE CDCONTADOR = 'LANCNF/FILIAL' || '0001';
               END IF;

               /*Tabela Lancamento Estoque*/
               SELECT LPAD(TO_CHAR(TO_NUMBER(NVL(MAX(NRSEQUENCIAL),'0')) + 1),10,'0')
               INTO V_NRLANCESTQ
               FROM TECNISA.NOVOCODIGO
               WHERE CDCONTADOR = 'LANCTOEST' || '0001';

               IF V_NRLANCTONF = '0000000001' THEN
                  INSERT INTO TECNISA.NOVOCODIGO
                     (CDCONTADOR, NRSEQUENCIAL)
                  VALUES
                     ('LANCTOEST' || '0001', LPAD(V_NRLANCESTQ,20,'0'));
               ELSE
                  UPDATE TECNISA.NOVOCODIGO
                  SET NRSEQUENCIAL = LPAD(V_NRLANCESTQ,20,'0')
                  WHERE CDCONTADOR = 'LANCTOEST' || '0001';
               END IF;

               /*Parmetros da filial*/
               SELECT MAX(CDEMPRESA), MAX(CDINSCESTA), MAX(CDINSCMUNI)
               INTO V_CDEMPRESA, V_CDINSCESTA, V_CDINSCMUNI
               FROM TECNISA.FILIAL
               WHERE CDFILIAL = '0001';

               /*Incluso do corpo do lanamento de nota fiscal*/
               INSERT INTO TECNISA.NOTAFISCAL
                  (CDFILIAL, NRLANCTONF, CDEMPRESA, CDINSCESTA, CDINSCMUNI,
                   CDFILIRECE, IDBLOQUEADO, DTENTRSAID, VRPRODUTOS, VRNOTAFISC,
                   IDENTRSAIDOP, CDTIPOOPER, IDSTATUS, IDSISTEMA, USR_RECB_SEQUENCIA)
               VALUES
                  ('0001', V_NRLANCTONF, V_CDEMPRESA, V_CDINSCESTA, V_CDINSCMUNI,
                   '0001', 'N', V_DTPROCESSA, 0, 0,
                   'E', '10', 'P', 'EST', :NEW.RECB_SEQUENCIA);

               /*Incluso do corpo do lanamento de lanamento de estoque*/
               INSERT INTO TECNISA.LANCTOESTOQ
                  (CDFILIAL, NRLANCESTQ, NRLANCTONF, IDTPLANCTO, DTLANCESTQ,
                   DSLANCESTQ, IDENTRSAIDOP, CDTIPOOPER, CDFILIMOVI, USR_RECB_SEQUENCIA)
               VALUES
                  ('0001', V_NRLANCESTQ, V_NRLANCTONF, '2', V_DTPROCESSA,
                   'Lanamento Estoque '||TO_CHAR(:NEW.RECB_SEQUENCIA), 'E', '10', '0001', :NEW.RECB_SEQUENCIA);

            ELSE
               /*Atualizao da data do corpo dos lanamentos*/
               UPDATE TECNISA.NOTAFISCAL
               SET DTENTRSAID = V_DTPROCESSA
               WHERE USR_RECB_SEQUENCIA = :NEW.RECB_SEQUENCIA;

               /*Atualizao da data do corpo dos lanamentos*/
               UPDATE TECNISA.LANCTOESTOQ
               SET DTLANCESTQ = V_DTPROCESSA
               WHERE USR_RECB_SEQUENCIA = :NEW.RECB_SEQUENCIA;
            END IF;

            V_VRTOTAITEM := :NEW.QT_VALIDADA * :NEW.VL_UNITARIO_NOTA_FISCAL;
            V_VRCONTITEM := V_VRTOTAITEM - ((V_VRTOTAITEM) * (NVL(:NEW.PC_DESCONTO,0)/100));

            /*Incluso de lote padro para produtos*/
            INSERT INTO TECNISA.LOTE
               (CDFILIAL, NRLOTEESTQ, NRSUBLOTE, CDPRODUTO, IDSTATLOTE, DTENTRLOTE)
               (SELECT '0001',  ' ', ' ',
                       V_CDPRODUTO, 'L' , SYSDATE
                FROM TECNISA.PARAMGERAL
                MINUS
                SELECT CDFILIAL, NRLOTEESTQ,  NRSUBLOTE,
                      V_CDPRODUTO, 'L', SYSDATE
                FROM TECNISA.LOTE
                WHERE  CDFILIAL   =  '0001'     AND
                       NRLOTEESTQ =  ' '        AND
                       NRSUBLOTE  =  ' '        AND
                       CDPRODUTO  =  V_CDPRODUTO);

            SELECT LPAD(TO_CHAR(TO_NUMBER(NVL(MAX(NRSEQITPROD),'0000')) + 1), 4, '0')
            INTO V_NRSEQITPROD
            FROM TECNISA.ITEMPRODUTO
            WHERE USR_RECB_SEQUENCIA = :NEW.RECB_SEQUENCIA;

            /*Incluso na tabela de itens da nota fiscal*/
            INSERT INTO TECNISA.ITEMPRODUTO
               (CDFILIAL, NRLANCTONF, NRSEQITPROD, CDPRODUTO, CDPRODEMIS,
                DSPRODEMIS, IDATUVRESTOQ, QTITEMPROD, VRUNITITEM, VRDESCITEM,
                VRTOTAITEM, VRCONTITEM, IDCONFORITEM, VRBRUTITEM, CDPRODMOVI,
                CDFILIMOVI, CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ, NRSUBLOTE,
                USR_SEQUENCIA, USR_RECB_SEQUENCIA, VRDESPERITEM)
            VALUES
               ('0001', V_NRLANCTONF, V_NRSEQITPROD, V_CDPRODUTO, V_CDPRODUTO,
                V_NMPRODUTO, 'S', :NEW.QT_VALIDADA, :NEW.VL_UNITARIO_NOTA_FISCAL, ((V_VRTOTAITEM) * (NVL(:NEW.PC_DESCONTO,0)/100)),
                V_VRTOTAITEM, V_VRCONTITEM, 'S', V_VRCONTITEM, V_CDPRODESTO,
                '0001', '98', ' ', ' ', ' ',
                :NEW.SEQUENCIA, :NEW.RECB_SEQUENCIA, NVL(:NEW.PC_DESCONTO,0));

            /*Incluso na tabela de itens de estoque*/
            INSERT INTO TECNISA.ITLANCTOEST
               (CDFILIAL, NRLANCESTQ, NRSEQUITEM, CDPRODUTO, QTLANCTOEST,
                VRUNILANCTO, VRTOTLANCTO, IDTIPOMOVI, DTLANCMOVI, IDORDEMCLC,
                VRLANCTOEST, QTTOTLANCTO, CDPRODMOVI, VRLANCTOBRUT, IDEVOLESTOQ,
                IDENTRSAIDOP, CDTIPOOPER, DSLANCESTIT, CDFILIMOVI, CDALMOXARIFE,
                CDLOCALESTOQ, VRCUSTENTR, NRLOTEESTQ, NRSUBLOTE, USR_SEQUENCIA,
                USR_RECB_SEQUENCIA)
            VALUES
               ('0001', V_NRLANCESTQ, V_NRSEQITPROD, V_CDPRODUTO, :NEW.QT_VALIDADA * V_VRFATOCONV,
                :NEW.VL_UNITARIO_NOTA_FISCAL, V_VRTOTAITEM, '2', V_DTPROCESSA, '2',
                V_VRCONTITEM/(:NEW.QT_VALIDADA * V_VRFATOCONV), :NEW.QT_VALIDADA, V_CDPRODESTO, V_VRCONTITEM/(:NEW.QT_VALIDADA * V_VRFATOCONV), 'S',
                'E', '10', 'Lanamento Estoque '||TO_CHAR(:NEW.RECB_SEQUENCIA), '0001', '98',
                ' ', V_VRCONTITEM, ' ', ' ', :NEW.SEQUENCIA,
                :NEW.RECB_SEQUENCIA);

            UPDATE TECNISA.PRODFILI
            SET DTULTCOMPR = V_DTPROCESSA,
                VRULTCOMPR = V_VRCONTITEM/:NEW.QT_VALIDADA
            WHERE CDFILIAL  = '0001'      AND
                  CDPRODUTO = V_CDPRODUTO AND
                  NVL(DTULTCOMPR, V_DTPROCESSA) <= V_DTPROCESSA;
         END IF;
      END IF;

      IF DELETING THEN
         /*Chave para posterior atualizao de estoque*/
         SELECT MAX(NRLANCESTQ)
         INTO V_NRLANCESTQ
         FROM TECNISA.ITLANCTOEST
         WHERE USR_RECB_SEQUENCIA = :OLD.RECB_SEQUENCIA;

         /*Excluso na tabela de itens da nota fiscal*/
         DELETE FROM TECNISA.ITEMPRODUTO
         WHERE USR_SEQUENCIA      = :OLD.SEQUENCIA AND
               USR_RECB_SEQUENCIA = :OLD.RECB_SEQUENCIA;

         /*Excluso na tabela de itens de estoque*/
         DELETE FROM TECNISA.ITLANCTOEST
         WHERE USR_SEQUENCIA      = :OLD.SEQUENCIA AND
               USR_RECB_SEQUENCIA = :OLD.RECB_SEQUENCIA;

         /*Verificao da existencia de itens no lanamento*/
         SELECT MAX(CDFILIAL)
         INTO V_CDFILIAL
         FROM TECNISA.ITLANCTOEST
         WHERE USR_RECB_SEQUENCIA = :OLD.RECB_SEQUENCIA;

         IF V_CDFILIAL IS NULL THEN
            /*Excluso do corpo do lanamento de nota fiscal*/
            DELETE TECNISA.NOTAFISCAL
            WHERE USR_RECB_SEQUENCIA = :OLD.RECB_SEQUENCIA;

            /*Excluso do corpo do lanamento de lanamento de estoque*/
            DELETE TECNISA.LANCTOESTOQ
            WHERE USR_RECB_SEQUENCIA = :OLD.RECB_SEQUENCIA;
         END IF;
      END IF;

      /*Atualizao de estoque de produtos*/
      TECNISA.ATUALIZA_ESTOQUE('0001', V_DTPROCESSA, V_DTPROCESSA, '0001', V_NRLANCESTQ);
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('TRG_INTERFACE_TECNISA','4.11.150'); END;
/

